@@ -18,11 +18,13 @@ unset($session); unset($request); unset($dbconn); unset($_awl_dbconn); unset($in
 // An ultra-simple exception handler to catch errors that occur
 // before we get a more functional exception handler in place...
 function early_exception_handler($e) {
-  echo "Uncaught early exception: ", $e->getMessage(), "\nAt line ", $e->getLine(), " of ", $e->getFile(), "\n";
+  echo "Exception [".$e->getCode()."] ".$e->getmessage()."\n";
+  echo "At line ", $e->getLine(), " of ", $e->getFile(), "\n";
+  echo "================= Stack Trace ===================\n";
 
   $trace = array_reverse($e->getTrace());
   foreach( $trace AS $k => $v ) {
-    printf( "=====================================================\n%s[%d] %s%s%s()\n", $v['file'], $v['line'], (isset($v['class'])?$v['class']:''), (isset($v['type'])?$v['type']:''), (isset($v['function'])?$v['function']:'') );
+    printf( "%s[%d] %s%s%s()\n", $v['file'], $v['line'], (isset($v['class'])?$v['class']:''), (isset($v['type'])?$v['type']:''), (isset($v['function'])?$v['function']:'') );
   }
 }
 set_exception_handler('early_exception_handler');
@@ -37,7 +39,8 @@ $c->collections_always_exist = false;
 $c->allow_get_email_visibility = false;
 $c->permission_scan_depth = 2;
 $c->expand_pdo_parameters = true;
-$c->home_calendar_name = 'home';
+$c->home_calendar_name    = 'calendar';
+$c->home_addressbook_name = 'addresses';
 $c->enable_row_linking = true;
 $c->enable_scheduling = false;
 $c->http_auth_mode = 'Basic';
@@ -74,7 +77,8 @@ $c->dbg = array();
 if ( ! @include_once('AWLUtilities.php') ) {
   $try_paths = array(
         '../../awl/inc'
-      , '/usr/share/awl/inc'
+      , '/usr/share/awl/inc'        // Where it ends up on Debian
+      , '/usr/share/php/awl/inc'    // Fedora's standard for PHP libraries
       , '/usr/local/share/awl/inc'
   );
   foreach( $try_paths AS $awl_include_path ) {
@@ -168,8 +172,8 @@ init_gettext( 'davical', $c->locale_path );
 *
 */
 $c->code_version = 0;
-$c->want_awl_version = 0.46;
-$c->version_string = '0.9.9.4'; // The actual version # is replaced into that during the build /release process
+$c->want_awl_version = '0.47';
+$c->version_string = '0.9.9.5'; // The actual version # is replaced into that during the build /release process
 if ( isset($c->version_string) && preg_match( '/(\d+)\.(\d+)\.(\d+)(.*)/', $c->version_string, $matches) ) {
   $c->code_major = $matches[1];
   $c->code_minor = $matches[2];
@@ -186,7 +190,7 @@ $_SERVER['SERVER_NAME'] = $c->domain_name;
 
 require_once('AwlQuery.php');
 
-$c->want_dbversion = array(1,2,9);
+$c->want_dbversion = array(1,2,10);
 $c->schema_version = 0;
 $qry = new AwlQuery( 'SELECT schema_major, schema_minor, schema_patch FROM awl_db_revision ORDER BY schema_id DESC LIMIT 1;' );
 if ( $qry->Exec('always',__LINE__,__FILE__) && $row = $qry->Fetch() ) {
@@ -202,92 +206,7 @@ if ( $qry->Exec('always',__LINE__,__FILE__) && $row = $qry->Fetch() ) {
   if ( isset($_SERVER['HTTP_X_DAVICAL_TESTCASE']) ) $qry->QDo('SET TIMEZONE TO \'Pacific/Auckland\'');
 }
 
-
-$_known_users_name  = array();
-$_known_users_id    = array();
-$_known_users_pid   = array();
-$_known_users_email = array();
-
-function _davical_get_principal_query_cached( $where, $parameter ) {
-  global $c, $session, $_known_users_name, $_known_users_id, $_known_users_pid;
-
-  $sql = 'SELECT *, to_char(updated at time zone \'GMT\',\'Dy, DD Mon IYYY HH24:MI:SS "GMT"\') AS modified, principal.*, ';
-  if ( isset($session->principal_id) ) {
-    $sql .= 'pprivs(:session_principal::int8,principal.principal_id,:scan_depth::int) AS privileges ';
-    $params = array( ':session_principal' => $session->principal_id, ':scan_depth' => $c->permission_scan_depth );
-  }
-  else {
-    $sql .= '0::BIT(24) AS privileges ';
-    $params = array( );
-  }
-  $sql .= 'FROM usr LEFT JOIN principal USING(user_no) WHERE '. $where;
-  $params[':param'] = $parameter;
-
-  $qry = new AwlQuery( $sql, $params );
-  if ( $qry->Exec('always',__LINE__,__FILE__) && $qry->rows() == 1 && $row = $qry->Fetch() ) {
-    if ( isset($session->principal_id) ) {
-      $_known_users_name[$row->username]      = $row;
-      $_known_users_id[$row->user_no]         = $row;
-      $_known_users_pid[$row->principal_id]   = $row;
-      $_known_users_email[$row->email]        = $row;
-    }
-    return $row;
-  }
-
-  return false;
-}
-
-/**
-* Return a user record identified by a username, caching it for any subsequent lookup
-* @param string $username The username of the record to retrieve
-* @param boolean $use_cache Whether or not to use the cache (default: yes)
-*/
-function getUserByName( $username, $use_cache = true ) {
-  global $_known_users_name;
-
-  if ( $use_cache && isset( $_known_users_name[$username] ) ) return $_known_users_name[$username];
-  return _davical_get_principal_query_cached( 'lower(username) = lower(:param)', $username );
-}
-
-
-/**
-* Return a user record identified by e-mail address, caching it for any subsequent lookup
-* @param string $email The email address of the user record to retrieve
-* @param boolean $use_cache Whether or not to use the cache (default: yes)
-*/
-function getUserByEMail( $email, $use_cache = true ) {
-  global $_known_users_name;
-
-  if ( $use_cache && isset( $_known_users_email[$email] ) ) return $_known_users_email[$email];
-  return _davical_get_principal_query_cached( 'lower(email) = lower(:param)', $email );
-}
-
-
-/**
-* Return a user record identified by a user_no, caching it for any subsequent lookup
-* @param int $user_no The ID of the record to retrieve
-* @param boolean $use_cache Whether or not to use the cache (default: yes)
-*/
-function getUserByID( $user_no, $use_cache = true ) {
-  global $c, $session, $_known_users_id;
-
-  if ( $use_cache && isset( $_known_users_id[$user_no] ) ) return $_known_users_id[$user_no];
-  return _davical_get_principal_query_cached( 'user_no = :param', $user_no );
-}
-
-
-/**
-* Return a user record identified by a user_no, caching it for any subsequent lookup
-* @param int $user_no The ID of the record to retrieve
-* @param boolean $use_cache Whether or not to use the cache (default: yes)
-*/
-function getPrincipalByID( $principal_id, $use_cache = true ) {
-  global $c, $session, $_known_users_pid;
-
-  if ( $use_cache && isset( $_known_users_pid[$principal_id] ) ) return $_known_users_pid[$principal_id];
-  return _davical_get_principal_query_cached( 'principal_id = :param', $principal_id );
-}
-
+require_once('Principal.php');
 
 /**
  * Return the HTTP status code description for a given code. Hopefully
@@ -420,8 +339,9 @@ function ISODateToHTTPDate( $isodate ) {
 * Convert a date into ISO format into the sparkly new ISO format.
 * @param string $indate The date to convert
 */
-function DateToISODate( $indate ) {
+function DateToISODate( $indate, $in_utc=false ) {
   // Use strtotime since strptime is not available on Windows platform.
+  if ( $in_utc ) return( gmdate('Ymd\THis\Z', strtotime($indate)) );
   return( date('c', strtotime($indate)) );
 }
 
