@@ -24,8 +24,35 @@ elseif ( isset($_SERVER['PATH_INFO']) && preg_match( '{^/\.well-known/(.+)$}', $
   exit(0);
 }
 require_once('./always.php');
-// dbg_error_log( 'caldav', ' User agent: %s', ((isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : 'Unfortunately Mulberry does not send a "User-agent" header with its requests :-(')) );
-// dbg_log_array( 'headers', '_SERVER', $_SERVER, true );
+
+function logRequestHeaders() {
+  global $c;
+  
+  /** Log the request headers */
+  $lines = apache_request_headers();
+  dbg_error_log( "LOG ", "***************** Request Header ****************" );
+  dbg_error_log( "LOG ", "%s %s", $_SERVER['REQUEST_METHOD'], $_SERVER['REQUEST_URI'] );
+  foreach( $lines AS $k => $v ) {
+    if ( $k != 'Authorization' || (isset($c->dbg['password']) && $c->dbg['password'] ) ) 
+      dbg_error_log( "LOG headers", "-->%s: %s", $k, $v );
+    else
+      dbg_error_log( "LOG headers", "-->%s: %s", $k, 'Delicious tasty password eaten by debugging monster!' );
+  }
+  dbg_error_log( "LOG ", "******************** Request ********************" );
+
+  // Log the request in all it's gory detail.
+  $lines = preg_split( '#[\r\n]+#', $c->raw_post );
+  foreach( $lines AS $v ) {
+    dbg_error_log( "LOG request", "-->%s", $v );
+  }
+  unset($lines);
+}
+
+if ( !isset($c->raw_post) ) $c->raw_post = file_get_contents( 'php://input');
+if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['request']) && $c->dbg['request']) )
+  logRequestHeaders();
+
+
 require_once('HTTPAuthSession.php');
 $session = new HTTPAuthSession();
 
@@ -66,7 +93,7 @@ $allowed = implode( ', ', array_keys($request->supported_methods) );
 if ( ! ($request->IsPrincipal() || isset($request->collection) || $request->method == 'PUT' || $request->method == 'MKCALENDAR' || $request->method == 'MKCOL' ) ) {
   if ( preg_match( '#^/principals/users(/.*/)$#', $request->path, $matches ) ) {
     // Although this doesn't work with the iPhone, perhaps it will with iCal...
-    /** @TODO: integrate handling this URL into CalDAVRequest.php */
+    /** @todo integrate handling this URL into CalDAVRequest.php */
     $redirect_url = ConstructURL('/caldav.php'.$matches[1]);
     dbg_error_log( 'LOG WARNING', 'Redirecting %s for "%s" to "%s"', $request->method, $request->path, $redirect_url );
     header('Location: '.$redirect_url );
@@ -86,8 +113,7 @@ switch ( $request->method ) {
     $request->CoerceContentType();
     switch( $request->content_type ) {
       case 'text/calendar':
-        /** use original DAViCal 'PUT' code which will be rewritten */
-        include('caldav-PUT.php');
+        include('caldav-PUT-vcalendar.php');
         break;
       case 'text/vcard':
       case 'text/x-vcard':
