@@ -14,8 +14,9 @@
 * @license   http://gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
 
+require_once("AwlCache.php");
 require_once("XMLDocument.php");
-require_once("CalDAVPrincipal.php");
+require_once("DAVPrincipal.php");
 include("DAVTicket.php");
 
 define('DEPTH_INFINITY', 9999);
@@ -47,7 +48,7 @@ class CalDAVRequest
 
   /**
   * The 'principal' (user/resource/...) which this request seeks to access
-  * @var CalDAVPrincipal
+  * @var DAVPrincipal
   */
   var $principal;
 
@@ -84,6 +85,11 @@ class CalDAVRequest
   */
   protected $exists;
 
+  /**
+  * The value of any 'Destionation:' header, if present.
+  */
+  var $destination;
+
   /**
   * The decimal privileges allowed by this user to the identified resource.
   */
@@ -105,59 +111,11 @@ class CalDAVRequest
   function __construct( $options = array() ) {
     global $session, $c, $debugging;
 
-    $this->supported_privileges = array(
-      'all' => array(
-        'read' => translate('Read the content of a resource or collection'),
-        'write' => array(
-          'bind' => translate('Create a resource or collection'),
-          'unbind' => translate('Delete a resource or collection'),
-          'write-content' => translate('Write content'),
-          'write-properties' => translate('Write properties')
-        ),
-        'urn:ietf:params:xml:ns:caldav:read-free-busy' => translate('Read the free/busy information for a calendar collection'),
-        'read-acl' => translate('Read ACLs for a resource or collection'),
-        'read-current-user-privilege-set' => translate('Read the details of the current user\'s access control to this resource.'),
-        'write-acl' => translate('Write ACLs for a resource or collection'),
-        'unlock' => translate('Remove a lock'),
-
-        'urn:ietf:params:xml:ns:caldav:schedule-deliver' => array(
-          'urn:ietf:params:xml:ns:caldav:schedule-deliver-invite'=> translate('Deliver scheduling invitations from an organiser to this scheduling inbox'),
-          'urn:ietf:params:xml:ns:caldav:schedule-deliver-reply' => translate('Deliver scheduling replies from an attendee to this scheduling inbox'),
-          'urn:ietf:params:xml:ns:caldav:schedule-query-freebusy' => translate('Allow free/busy enquiries targeted at the owner of this scheduling inbox')
-        ),
-
-        'urn:ietf:params:xml:ns:caldav:schedule-send' => array(
-          'urn:ietf:params:xml:ns:caldav:schedule-send-invite' => translate('Send scheduling invitations as an organiser from the owner of this scheduling outbox.'),
-          'urn:ietf:params:xml:ns:caldav:schedule-send-reply' => translate('Send scheduling replies as an attendee from the owner of this scheduling outbox.'),
-          'urn:ietf:params:xml:ns:caldav:schedule-send-freebusy' => translate('Send free/busy enquiries')
-        )
-      )
-    );
-
     $this->options = $options;
     if ( !isset($this->options['allow_by_email']) ) $this->options['allow_by_email'] = false;
-    $this->principal = (object) array( 'username' => $session->username, 'user_no' => $session->user_no );
 
-    $this->raw_post = file_get_contents ( 'php://input');
-
-    if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['request']) && $c->dbg['request']) ) {
-      /** Log the request headers */
-      $lines = apache_request_headers();
-      dbg_error_log( "LOG ", "***************** Request Header ****************" );
-      dbg_error_log( "LOG ", "%s %s", $_SERVER['REQUEST_METHOD'], $_SERVER['REQUEST_URI'] );
-      foreach( $lines AS $k => $v ) {
-        if ( $k == 'Authorization' )
-          dbg_error_log( "LOG headers", "-->%s: %s", $k, 'Delicious tasty password eaten by debugging monster!' );
-        else
-        dbg_error_log( "LOG headers", "-->%s: %s", $k, $v );
-      }
-      dbg_error_log( "LOG ", "******************** Request ********************" );
-      // Log the request in all it's gory detail.
-      $lines = preg_split( '#[\r\n]+#', $this->raw_post);
-      foreach( $lines AS $v ) {
-        dbg_error_log( "LOG request", "-->%s", $v );
-      }
-    }
+    if ( !isset($c->raw_post) ) $c->raw_post = file_get_contents( 'php://input');
+    $this->raw_post = $c->raw_post;
 
     if ( isset($debugging) && isset($_GET['method']) ) {
       $_SERVER['REQUEST_METHOD'] = $_GET['method'];
@@ -173,7 +131,7 @@ class CalDAVRequest
       }
       if ( $this->method == 'PROPFIND' || $this->method == 'REPORT' ) {
         if ( !preg_match( '{^(text|application)/xml$}', $this->content_type ) ) {
-          dbg_error_log( "LOG request", 'Request is "%s" but client set content-type to "%s". Assuming they meant XML!',
+          @dbg_error_log( "LOG request", 'Request is "%s" but client set content-type to "%s". Assuming they meant XML!',
                                                  $request->method, $this->content_type );
           $this->content_type = 'text/xml';
         }
@@ -213,7 +171,12 @@ class CalDAVRequest
     /**
     * MOVE/COPY use a "Destination" header and (optionally) an "Overwrite" one.
     */
-    if ( isset($_SERVER['HTTP_DESTINATION']) ) $this->destination = $_SERVER['HTTP_DESTINATION'];
+    if ( isset($_SERVER['HTTP_DESTINATION']) ) {
+      $this->destination = $_SERVER['HTTP_DESTINATION'];
+      if ( preg_match('{^(https?)://([a-z.-]+)(:[0-9]+)?(/.*)$}', $this->destination, $matches ) ) {
+        $this->destination = $matches[4];
+      }
+    }
     $this->overwrite = ( isset($_SERVER['HTTP_OVERWRITE']) && ($_SERVER['HTTP_OVERWRITE'] == 'F') ? false : true ); // RFC4918, 9.8.4 says default True.
 
     /**
@@ -280,14 +243,7 @@ class CalDAVRequest
     }
     if ( strstr($this->path,'//') ) $this->path = preg_replace( '#//+#', '/', $this->path);
 
-    $this->user_no = $session->user_no;
-    $this->username = $session->username;
-    if ( $session->user_no > 0 ) {
-      $this->current_user_principal_url = new XMLElement('href', ConstructURL('/'.$session->username.'/') );
-    }
-    else {
-      $this->current_user_principal_url = new XMLElement('unauthenticated' );
-    }
+    $this->principal = new Principal('path',$this->path);
 
     /**
     * RFC2518, 5.2: URL pointing to a collection SHOULD end in '/', and if it does not then
@@ -343,6 +299,11 @@ EOSQL;
       $qry->Exec('caldav',__LINE__,__FILE__);
       dbg_error_log( 'caldav', 'Created new collection as "%s".', trim($params[':boxname']) );
 
+      // Uncache anything to do with the collection
+      $cache = getCacheInstance();
+      $cache->delete( 'collection-'.$params[':dav_name'], null );
+      $cache->delete( 'principal-'.$params[':parent_container'], null );
+      
       $qry = new AwlQuery( "SELECT * FROM collection WHERE dav_name = :dav_name", array( ':dav_name' => $matches[1] ) );
       if ( $qry->Exec('caldav',__LINE__,__FILE__) && $qry->rows() == 1 && ($row = $qry->Fetch()) ) {
         $this->collection_id = $row->collection_id;
@@ -363,7 +324,7 @@ EOSQL;
       }
     }
     else if ( $this->options['allow_by_email'] && preg_match( '#^/(\S+@\S+[.]\S+)/?$#', $this->path) ) {
-      /** @TODO: we should deprecate this now that Evolution 2.27 can do scheduling extensions */
+      /** @todo we should deprecate this now that Evolution 2.27 can do scheduling extensions */
       $this->collection_id = -1;
       $this->collection_type = 'email';
       $this->collection_path = $this->path;
@@ -396,11 +357,11 @@ EOSQL;
     /**
     * Extract the user whom we are accessing
     */
-    $this->principal = new CalDAVPrincipal( array( "path" => $this->path, "options" => $this->options ) );
-    if ( isset($this->principal->user_no) ) $this->user_no  = $this->principal->user_no;
-    if ( isset($this->principal->username)) $this->username = $this->principal->username;
-    if ( isset($this->principal->by_email) && $this->principal->by_email) $this->by_email = true;
-    if ( isset($this->principal->principal_id)) $this->principal_id = $this->principal->principal_id;
+    $this->principal = new DAVPrincipal( array( "path" => $this->path, "options" => $this->options ) );
+    $this->user_no  = $this->principal->user_no();
+    $this->username = $this->principal->username();
+    $this->by_email = $this->principal->byEmail();
+    $this->principal_id = $this->principal->principal_id();
 
     if ( $this->collection_type == 'principal' || $this->collection_type == 'email' || $this->collection_type == 'proxy' ) {
       $this->collection = $this->principal->AsCollection();
@@ -556,45 +517,6 @@ EOSQL;
   }
 
 
-  /**
-  * Work out the user whose calendar we are accessing, based on elements of the path.
-  */
-  function UserFromPath() {
-    global $session;
-
-    $this->user_no = $session->user_no;
-    $this->username = $session->username;
-    $this->principal_id = $session->principal_id;
-
-    @dbg_error_log( "WARN", "Call to deprecated CalDAVRequest::UserFromPath()" );
-
-    if ( $this->path == '/' || $this->path == '' ) {
-      dbg_error_log( "caldav", "No useful path split possible" );
-      return false;
-    }
-
-    $path_split = explode('/', $this->path );
-    $this->username = $path_split[1];
-    if ( $this->username == 'principals' ) $this->username = $path_split[3];
-    @dbg_error_log( "caldav", "Path split into at least /// %s /// %s /// %s", $path_split[1], $path_split[2], $path_split[3] );
-    if ( isset($this->options['allow_by_email']) && preg_match( '#/(\S+@\S+[.]\S+)/?$#', $this->path, $matches) ) {
-      $this->by_email = $matches[1];
-      $qry = new AwlQuery("SELECT user_no, principal_id, username FROM usr JOIN principal USING (user_no) WHERE email = :email",
-                          array(':email' => $this->by_email ) );
-      if ( $qry->Exec('caldav',__LINE__,__FILE__) && $user = $qry->Fetch() ) {
-        $this->user_no = $user->user_no;
-        $this->username = $user->username;
-        $this->principal_id = $user->principal_id;
-      }
-    }
-    elseif( $user = getUserByName($this->username,'caldav',__LINE__,__FILE__)) {
-      $this->principal = $user;
-      $this->user_no = $user->user_no;
-      $this->principal_id = $user->principal_id;
-    }
-  }
-
-
   /**
   * Permissions are controlled as follows:
   *  1. if the path is '/', the request has read privileges
@@ -614,9 +536,9 @@ EOSQL;
       $this->privileges = privilege_to_bits( array('read','read-free-busy','read-acl'));
       dbg_error_log( "caldav", "Full read permissions for user accessing /" );
     }
-    else if ( $session->AllowedTo("Admin") || $session->user_no == $this->user_no ) {
+    else if ( $session->AllowedTo("Admin") || $session->principal->user_no() == $this->user_no ) {
       $this->privileges = privilege_to_bits('all');
-      dbg_error_log( "caldav", "Full permissions for %s", ( $session->user_no == $this->user_no ? "user accessing their own hierarchy" : "a systems administrator") );
+      dbg_error_log( "caldav", "Full permissions for %s", ( $session->principal->user_no() == $this->user_no ? "user accessing their own hierarchy" : "a systems administrator") );
     }
     else {
       $this->privileges = 0;
@@ -632,7 +554,7 @@ EOSQL;
       /**
       * In other cases we need to query the database for permissions
       */
-      $params = array( ':session_principal_id' => $session->principal_id, ':scan_depth' => $c->permission_scan_depth );
+      $params = array( ':session_principal_id' => $session->principal->principal_id(), ':scan_depth' => $c->permission_scan_depth );
       if ( isset($this->by_email) && $this->by_email ) {
         $sql ='SELECT pprivs( :session_principal_id::int8, :request_principal_id::int8, :scan_depth::int ) AS perm';
         $params[':request_principal_id'] = $this->principal_id;
@@ -720,6 +642,37 @@ EOSQL;
   }
 
 
+  private static function supportedPrivileges() { 
+    return array(
+      'all' => array(
+        'read' => translate('Read the content of a resource or collection'),
+        'write' => array(
+          'bind' => translate('Create a resource or collection'),
+          'unbind' => translate('Delete a resource or collection'),
+          'write-content' => translate('Write content'),
+          'write-properties' => translate('Write properties')
+        ),
+        'urn:ietf:params:xml:ns:caldav:read-free-busy' => translate('Read the free/busy information for a calendar collection'),
+        'read-acl' => translate('Read ACLs for a resource or collection'),
+        'read-current-user-privilege-set' => translate('Read the details of the current user\'s access control to this resource.'),
+        'write-acl' => translate('Write ACLs for a resource or collection'),
+        'unlock' => translate('Remove a lock'),
+
+        'urn:ietf:params:xml:ns:caldav:schedule-deliver' => array(
+          'urn:ietf:params:xml:ns:caldav:schedule-deliver-invite'=> translate('Deliver scheduling invitations from an organiser to this scheduling inbox'),
+          'urn:ietf:params:xml:ns:caldav:schedule-deliver-reply' => translate('Deliver scheduling replies from an attendee to this scheduling inbox'),
+          'urn:ietf:params:xml:ns:caldav:schedule-query-freebusy' => translate('Allow free/busy enquiries targeted at the owner of this scheduling inbox')
+        ),
+
+        'urn:ietf:params:xml:ns:caldav:schedule-send' => array(
+          'urn:ietf:params:xml:ns:caldav:schedule-send-invite' => translate('Send scheduling invitations as an organiser from the owner of this scheduling outbox.'),
+          'urn:ietf:params:xml:ns:caldav:schedule-send-reply' => translate('Send scheduling replies as an attendee from the owner of this scheduling outbox.'),
+          'urn:ietf:params:xml:ns:caldav:schedule-send-freebusy' => translate('Send free/busy enquiries')
+        )
+      )
+    );
+  }
+  
   /**
   * Returns the dav_name of the resource in our internal namespace
   */
@@ -887,6 +840,7 @@ EOSQL;
         dbg_error_log( 'LOG NOTICE', 'Unusual content-type of "%s" and first word of content is "%s"',
                                         (isset($this->content_type)?$this->content_type:'(null)'), $first_word );
     }
+    $this->content_type = 'text/plain';
   }
 
 
@@ -962,7 +916,7 @@ EOSQL;
   */
   function BuildSupportedPrivileges( &$reply, $privs = null ) {
     $privileges = array();
-    if ( $privs === null ) $privs = $this->supported_privileges;
+    if ( $privs === null ) $privs = self::supportedPrivileges();
     foreach( $privs AS $k => $v ) {
       dbg_error_log( 'caldav', 'Adding privilege "%s" which is "%s".', $k, $v );
       $privilege = new XMLElement('privilege');
@@ -1000,7 +954,7 @@ EOSQL;
   */
   function AllowedTo( $activity ) {
     global $session;
-    dbg_error_log('caldav', 'Checking whether "%s" is allowed to "%s"', $session->username, $activity);
+    dbg_error_log('caldav', 'Checking whether "%s" is allowed to "%s"', $session->principal->username(), $activity);
     if ( isset($this->permissions['all']) ) return true;
     switch( $activity ) {
       case 'all':
@@ -1136,7 +1090,7 @@ EOSQL;
     $xmldoc = sprintf('<?xml version="1.0" encoding="utf-8" ?>
 <error xmlns="DAV:">
   <%s/>%s
-</error>', $precondition, $explanation );
+</error>', str_replace('DAV::', '', $precondition), $explanation );
 
     $this->DoResponse( $status, $xmldoc, 'text/xml; charset="utf-8"' );
     exit(0);  // Unecessary, but might clarify things
@@ -1203,7 +1157,7 @@ EOSQL;
         $message = substr( preg_replace("#\s+#m", ' ', $message ), 0, 100) . (strlen($message) > 100 ? "..." : "");
       }
 
-      dbg_error_log("caldav", "Status: %d, Message: %s, User: %d, Path: %s", $status, $message, $session->user_no, $this->path);
+      dbg_error_log("caldav", "Status: %d, Message: %s, User: %d, Path: %s", $status, $message, $session->principal->user_no(), $this->path);
     }
     if ( isset($c->dbg['statistics']) && $c->dbg['statistics'] ) {
       $script_time = microtime(true) - $c->script_start_time;
