@@ -9,8 +9,10 @@
 * @license   http://gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
 
+require_once('AwlCache.php');
 require_once('AwlQuery.php');
 require_once('DAVTicket.php');
+require_once('iCalendar.php');
 
 
 /**
@@ -95,6 +97,11 @@ class DAVResource
   */
   private $_is_binding;
 
+  /**
+  * @var True if this resource is a binding to an external resource
+  */
+  private $_is_external;
+
   /**
   * @var True if this resource is an addressbook collection
   */
@@ -156,6 +163,7 @@ class DAVResource
     $this->_is_principal     = false;
     $this->_is_calendar      = false;
     $this->_is_binding       = false;
+    $this->_is_external      = false;
     $this->_is_addressbook   = false;
     $this->_is_proxy_request = false;
     if ( isset($parameters) && is_object($parameters) ) {
@@ -213,10 +221,13 @@ class DAVResource
       switch ( $k ) {
         case 'created':
         case 'modified':
-        case 'resourcetypes':
           $this->{$k} = $v;
           break;
 
+        case 'resourcetypes':
+          if ( $this->_is_collection ) $this->{$k} = $v;
+          break;
+
         case 'dav_etag':
           $this->unique_tag = '"'.$v.'"';
           break;
@@ -245,11 +256,11 @@ class DAVResource
           $this->collection->type = 'collection';
       }
 
-      $this->_is_calendar     = ($this->collection->is_calendar == 't');
-      $this->_is_addressbook  = ($this->collection->is_addressbook == 't');
-      $this->_is_proxy_request= ($this->collection->type == 'proxy');
+      $this->_is_calendar      = ($this->collection->is_calendar == 't');
+      $this->_is_addressbook   = ($this->collection->is_addressbook == 't');
+      $this->_is_proxy_request = ($this->collection->type == 'proxy');
       if ( $this->_is_principal && !isset($this->resourcetypes) ) {
-        $this->resourcetypes = '<DAV::collection/><DAV::principal/>';
+        $this->resourcetypes   = '<DAV::collection/><DAV::principal/>';
       }
       else if ( $this->_is_proxy_request ) {
         $this->resourcetypes  = $this->collection->resourcetypes;
@@ -259,8 +270,18 @@ class DAVResource
     else {
       $this->resourcetypes = '';
       if ( isset($this->resource->caldav_data) ) {
-        if ( substr($this->resource->caldav_data,0,15) == 'BEGIN:VCALENDAR' ) {
+        $this->resource->displayname = $this->resource->summary;
+        if ( strtoupper(substr($this->resource->caldav_data,0,15)) == 'BEGIN:VCALENDAR' ) {
           $this->contenttype = 'text/calendar';
+          if ( !$this->HavePrivilegeTo('read') && $this->HavePrivilegeTo('read-free-busy') ) {
+            $vcal = new iCalComponent($this->resource->caldav_data);
+            $confidential = $vcal->CloneConfidential();
+            $this->resource->caldav_data = $confidential->Render();
+            $this->resource->displayname = $this->resource->summary = translate('Busy');
+            $this->resource->description = null;
+            $this->resource->location = null;
+            $this->resource->url = null;
+          }
         }
         else if ( strtoupper(substr($this->resource->caldav_data,0,11)) == 'BEGIN:VCARD' ) {
           $this->contenttype = 'text/vcard';
@@ -268,7 +289,6 @@ class DAVResource
         else if ( strtoupper(substr($this->resource->caldav_data,0,11)) == 'BEGIN:VLIST' ) {
           $this->contenttype = 'text/x-vlist';
         }
-        $this->resource->displayname = $this->resource->summary;
       }
     }
   }
@@ -295,23 +315,8 @@ class DAVResource
   }
 
 
-  /**
-  * Find the collection associated with this resource.
-  */
-  function FetchCollection() {
-    global $c, $session, $request;
-    /**
-    * RFC4918, 8.3: Identifiers for collections SHOULD end in '/'
-    *    - also discussed at more length in 5.2
-    *
-    * So we look for a collection which matches one of the following URLs:
-    *  - The exact request.
-    *  - If the exact request, doesn't end in '/', then the request URL with a '/' appended
-    *  - The request URL truncated to the last '/'
-    * The collection URL for this request is therefore the longest row in the result, so we
-    * can "... ORDER BY LENGTH(dav_name) DESC LIMIT 1"
-    */
-    dbg_error_log( 'DAVResource', ':FetchCollection: Looking for collection for "%s".', $this->dav_name );
+  private function ReadCollectionFromDatabase() {
+    global $c, $session;
 
     $this->collection = (object) array(
       'collection_id' => -1,
@@ -385,7 +390,9 @@ EOSQL;
            || preg_match( '#^((/principals/[^/]+/)[^/]+)/?$#', $this->dav_name, $matches) ) {
       $this->_is_principal = true;
       $this->FetchPrincipal();
-    }
+      $this->collection->is_principal = true;
+      $this->collection->type = 'principal';
+   }
     else if ( $this->dav_name == '/' ) {
       $this->collection->dav_name = '/';
       $this->collection->type = 'root';
@@ -399,7 +406,8 @@ EOSQL;
 SELECT collection.*, path_privs(:session_principal::int8, collection.dav_name,:scan_depth::int), p.principal_id,
     p.type_id AS principal_type_id, p.displayname AS principal_displayname, p.default_privileges AS principal_default_privileges,
     time_zone.tz_spec, dav_binding.access_ticket_id, dav_binding.parent_container AS bind_parent_container,
-    dav_binding.dav_displayname, owner.dav_name AS bind_owner_url, dav_binding.dav_name AS bound_to
+		dav_binding.dav_displayname, owner.dav_name AS bind_owner_url, dav_binding.dav_name AS bound_to,  
+    dav_binding.external_url AS external_url, dav_binding.type AS external_type, dav_binding.bind_id AS bind_id 
 FROM dav_binding
     LEFT JOIN collection ON (collection.collection_id=bound_source_id)
     LEFT JOIN principal p USING (user_no)
@@ -418,16 +426,10 @@ EOSQL;
       if ( $qry->Exec('DAVResource',__LINE__,__FILE__) && $qry->rows() == 1 && ($row = $qry->Fetch()) ) {
         $this->collection = $row;
         $this->collection->exists = true;
-        $this->_is_binding = true;
         $this->collection->parent_set = $row->parent_container;
         $this->collection->parent_container = $row->bind_parent_container;
-        $this->bound_from = str_replace( $row->bound_to, $row->dav_name, $this->dav_name);
         $this->collection->bound_from = $row->dav_name;
         $this->collection->dav_name = $row->bound_to;
-        if ( isset($row->access_ticket_id) ) {
-          if ( !isset($this->tickets) ) $this->tickets = array();
-          $this->tickets[] = new DAVTicket($row->access_ticket_id);
-        }
         if ( $row->is_calendar == 't' )
           $this->collection->type = 'calendar';
         else if ( $row->is_addressbook == 't' )
@@ -436,6 +438,22 @@ EOSQL;
           $this->collection->type = 'schedule-'. $matches[3]. 'box';
         else
           $this->collection->type = 'collection';
+        if ( strlen($row->external_url) > 8 ) {
+          $this->collection->bound_from = $row->external_url;
+          $this->_is_external = true;
+          if ( $row->external_type == 'calendar' )
+            $this->collection->type = 'calendar';
+          else if ( $row->external_type == 'addressbook' )
+            $this->collection->type = 'addressbook';
+          else
+            $this->collection->type = 'collection';
+        }
+        $this->_is_binding = true;
+        $this->bound_from = str_replace( $row->bound_to, $row->dav_name, $this->dav_name);
+        if ( isset($row->access_ticket_id) ) {
+          if ( !isset($this->tickets) ) $this->tickets = array();
+          $this->tickets[] = new DAVTicket($row->access_ticket_id);
+        }
       }
       else {
         dbg_error_log( 'DAVResource', 'No collection for path "%s".', $this->dav_name );
@@ -443,9 +461,63 @@ EOSQL;
         $this->collection->dav_name = preg_replace('{/[^/]*$}', '/', $this->dav_name);
       }
     }
+    
+  }
+  
+  /**
+  * Find the collection associated with this resource.
+  */
+  protected function FetchCollection() {
+    global $session;
+    
+    /**
+    * RFC4918, 8.3: Identifiers for collections SHOULD end in '/'
+    *    - also discussed at more length in 5.2
+    *
+    * So we look for a collection which matches one of the following URLs:
+    *  - The exact request.
+    *  - If the exact request, doesn't end in '/', then the request URL with a '/' appended
+    *  - The request URL truncated to the last '/'
+    * The collection URL for this request is therefore the longest row in the result, so we
+    * can "... ORDER BY LENGTH(dav_name) DESC LIMIT 1"
+    */
+    dbg_error_log( 'DAVResource', ':FetchCollection: Looking for collection for "%s".', $this->dav_name );
 
-    @dbg_error_log( 'DAVResource', ':FetchCollection: Found collection named "%s" of type "%s".', $this->collection->dav_name, $this->collection->type );
-
+    // Try and pull the answer out of a hat
+    $cache = getCacheInstance();
+    $cache_ns = 'collection-'.preg_replace( '{/[^/]*$}', '/', $this->dav_name);
+    $cache_key = 'dav_resource'.$session->user_no;
+    $this->collection = $cache->get( $cache_ns, $cache_key );
+    if ( $this->collection === false ) {
+      $this->ReadCollectionFromDatabase();
+      if ( $this->collection->type != 'principal' ) {
+        $cache_ns = 'collection-'.$this->collection->dav_name;
+        @dbg_error_log( 'Cache', ':FetchCollection: Setting cache ns "%s" key "%s". Type: %s', $cache_ns, $cache_key, $this->collection->type );
+        $cache->set( $cache_ns, $cache_key, $this->collection );
+      }
+      @dbg_error_log( 'DAVResource', ':FetchCollection: Found collection named "%s" of type "%s".', $this->collection->dav_name, $this->collection->type );
+    }
+    else {
+      @dbg_error_log( 'Cache', ':FetchCollection: Got cache ns "%s" key "%s". Type: %s', $cache_ns, $cache_key, $this->collection->type );
+      if ( preg_match( '#^(/[^/]+)/?$#', $this->dav_name, $matches)
+           || preg_match( '#^((/principals/[^/]+/)[^/]+)/?$#', $this->dav_name, $matches) ) {
+        $this->_is_principal = true;
+        $this->FetchPrincipal();
+        $this->collection->is_principal = true;
+        $this->collection->type = 'principal';
+      }
+      @dbg_error_log( 'DAVResource', ':FetchCollection: Read cached collection named "%s" of type "%s".', $this->collection->dav_name, $this->collection->type );
+    }
+    
+    if ( isset($this->collection->bound_from) ) {
+      $this->_is_binding = true;
+      $this->bound_from = str_replace( $this->collection->bound_to, $this->collection->bound_from, $this->dav_name);
+      if ( isset($this->collection->access_ticket_id) ) {
+        if ( !isset($this->tickets) ) $this->tickets = array();
+        $this->tickets[] = new DAVTicket($this->collection->access_ticket_id);
+      }
+    }
+    
     $this->_is_collection = ( $this->_is_principal || $this->collection->dav_name == $this->dav_name || $this->collection->dav_name == $this->dav_name.'/' );
     if ( $this->_is_collection ) {
       $this->dav_name = $this->collection->dav_name;
@@ -464,7 +536,7 @@ EOSQL;
         if ( isset($this->collection->dav_displayname) ) $this->collection->displayname = $this->collection->dav_displayname;
       }
       else {
-        if ( !isset($this->parent) ) $this->FetchParentContainer();
+        if ( !isset($this->parent) ) $this->GetParentContainer();
         $this->user_no = $this->parent->GetProperty('user_no');
       }
       if ( isset($this->collection->resourcetypes) )
@@ -482,22 +554,21 @@ EOSQL;
   /**
   * Find the principal associated with this resource.
   */
-  function FetchPrincipal() {
+  protected function FetchPrincipal() {
     if ( isset($this->principal) ) return;
-    $this->principal = new CalDAVPrincipal( array( "path" => $this->bound_from() ) );
+    $this->principal = new DAVPrincipal( array( "path" => $this->bound_from() ) );
     if ( $this->_is_principal ) {
       $this->exists = $this->principal->Exists();
-      $this->collection->dav_name = $this->dav_name;
+      $this->collection->dav_name = $this->dav_name();
       $this->collection->type = 'principal';
       if ( $this->exists ) {
+        $this->collection = $this->principal->AsCollection();
         $this->displayname = $this->principal->GetProperty('displayname');
-        $this->unique_tag = '"'.$this->principal->dav_etag.'"';
+        $this->user_no = $this->principal->user_no();
+        $this->resource_id = $this->principal->principal_id();
         $this->created = $this->principal->created;
         $this->modified = $this->principal->modified;
-        $this->resourcetypes = '<DAV::collection/><DAV::principal/>';
-        $this->resource_id = $this->principal->principal_id;
-        $this->collection = $this->principal->AsCollection();
-        $this->user_no = $this->principal->user_no;
+        $this->resourcetypes = $this->principal->resourcetypes;
       }
     }
   }
@@ -506,7 +577,7 @@ EOSQL;
   /**
   * Retrieve the actual resource.
   */
-  function FetchResource() {
+  protected function FetchResource() {
     global $c, $session;
 
     if ( isset($this->exists) ) return;   // True or false, we've got what we can already
@@ -535,10 +606,12 @@ EOQRY;
   /**
   * Fetch any dead properties for this URL
   */
-  function FetchDeadProperties() {
+  protected function FetchDeadProperties() {
     if ( isset($this->dead_properties) ) return;
 
     $this->dead_properties = array();
+    if ( !$this->exists || !$this->_is_collection ) return;
+
     $qry = new AwlQuery('SELECT property_name, property_value FROM property WHERE dav_name= :dav_name', array(':dav_name' => $this->dav_name) );
     if ( $qry->Exec('DAVResource') ) {
       while ( $property = $qry->Fetch() ) {
@@ -549,9 +622,67 @@ EOQRY;
 
 
   /**
-  * Fetch the parent to this resource.
+  * Build permissions for this URL
   */
-  function FetchParentContainer() {
+  protected function FetchPrivileges() {
+    global $session, $request;
+
+    if ( $this->dav_name == '/' || $this->dav_name == '' ) {
+      $this->privileges = (1 | 16 | 32); // read + read-acl + read-current-user-privilege-set
+      dbg_error_log( 'DAVResource', ':FetchPrivileges: Read permissions for user accessing /' );
+      return;
+    }
+
+    if ( $session->AllowedTo('Admin') ) {
+      $this->privileges = privilege_to_bits('all');
+      dbg_error_log( 'DAVResource', ':FetchPrivileges: Full permissions for an administrator.' );
+      return;
+    }
+
+    if ( $this->IsPrincipal() ) {
+      if ( !isset($this->principal) ) $this->FetchPrincipal();
+      $this->privileges = $this->principal->Privileges();
+      dbg_error_log( 'DAVResource', ':FetchPrivileges: Privileges of "%s" for user accessing principal "%s"', $this->privileges, $this->principal->username() );
+      return;
+    }
+
+    if ( ! isset($this->collection) ) $this->FetchCollection();
+    $this->privileges = 0;
+    if ( !isset($this->collection->path_privs) ) {
+      if ( !isset($this->parent) ) $this->GetParentContainer();
+
+      $this->collection->path_privs = $this->parent->Privileges();
+      $this->collection->user_no = $this->parent->GetProperty('user_no');
+      $this->collection->principal_id = $this->parent->GetProperty('principal_id');
+    }
+
+    $this->privileges = $this->collection->path_privs;
+    if ( is_string($this->privileges) ) $this->privileges = bindec( $this->privileges );
+
+    dbg_error_log( 'DAVResource', ':FetchPrivileges: Privileges of "%s" for user "%s" accessing "%s"',
+                       decbin($this->privileges), $session->username, $this->dav_name() );
+
+    if ( isset($request->ticket) && $request->ticket->MatchesPath($this->bound_from()) ) {
+      $this->privileges |= $request->ticket->privileges();
+      dbg_error_log( 'DAVResource', ':FetchPrivileges: Applying permissions for ticket "%s" now: %s', $request->ticket->id(), decbin($this->privileges) );
+    }
+
+    if ( isset($this->tickets) ) {
+      if ( !isset($this->resource_id) ) $this->FetchResource();
+      foreach( $this->tickets AS $k => $ticket ) {
+        if ( $ticket->MatchesResource($this->resource_id()) || $ticket->MatchesPath($this->bound_from()) ) {
+          $this->privileges |= $ticket->privileges();
+          dbg_error_log( 'DAVResource', ':FetchPrivileges: Applying permissions for ticket "%s" now: %s', $ticket->id(), decbin($this->privileges) );
+        }
+      }
+    }
+  }
+
+
+  /**
+  * Get a DAVResource which is the parent to this resource.
+  */
+  function GetParentContainer() {
     if ( $this->dav_name == '/' ) return null;
     if ( !isset($this->parent) ) {
       if ( $this->_is_collection ) {
@@ -566,62 +697,13 @@ EOQRY;
     return $this->parent;
   }
 
-
+  
   /**
-  * Build permissions for this URL
+  * Fetch the parent to this resource.
   */
-  function FetchPrivileges() {
-    global $session, $request;
-
-    if ( $this->dav_name == '/' || $this->dav_name == '' ) {
-      $this->privileges = (1 | 16 | 32); // read + read-acl + read-current-user-privilege-set
-      dbg_error_log( 'DAVResource', 'Read permissions for user accessing /' );
-      return;
-    }
-
-    if ( $session->AllowedTo('Admin') ) {
-      $this->privileges = privilege_to_bits('all');
-      dbg_error_log( 'DAVResource', 'Full permissions for an administrator.' );
-      return;
-    }
-
-    if ( $this->IsPrincipal() ) {
-      if ( !isset($this->principal) ) $this->FetchPrincipal();
-      $this->privileges = $this->principal->Privileges();
-      dbg_error_log( 'DAVResource', 'Privileges of "%s" for user accessing principal "%s"', $this->privileges, $this->principal->username() );
-      return;
-    }
-
-    if ( ! isset($this->collection) ) $this->FetchCollection();
-    $this->privileges = 0;
-    if ( !isset($this->collection->path_privs) ) {
-      if ( !isset($this->parent) ) $this->FetchParentContainer();
-
-      $this->collection->path_privs = $this->parent->Privileges();
-      $this->collection->user_no = $this->parent->GetProperty('user_no');
-      $this->collection->principal_id = $this->parent->GetProperty('principal_id');
-    }
-
-    $this->privileges = $this->collection->path_privs;
-    if ( is_string($this->privileges) ) $this->privileges = bindec( $this->privileges );
-
-    dbg_error_log( 'DAVResource', 'Privileges of "%s" for user "%s" accessing "%s"',
-                       decbin($this->privileges), $session->username, $this->dav_name() );
-
-    if ( isset($request->ticket) && $request->ticket->MatchesPath($this->bound_from()) ) {
-      $this->privileges |= $request->ticket->privileges();
-      dbg_error_log( 'DAVResource', 'Applying permissions for ticket "%s" now: %s', $request->ticket->id(), decbin($this->privileges) );
-    }
-
-    if ( isset($this->tickets) ) {
-      if ( !isset($this->resource_id) ) $this->FetchResource();
-      foreach( $this->tickets AS $k => $ticket ) {
-        if ( $ticket->MatchesResource($this->resource_id()) || $ticket->MatchesPath($this->bound_from()) ) {
-          $this->privileges |= $ticket->privileges();
-          dbg_error_log( 'DAVResource', 'Applying permissions for ticket "%s" now: %s', $ticket->id(), decbin($this->privileges) );
-        }
-      }
-    }
+  function FetchParentContainer() {
+    deprecated('DAVResource::FetchParentContainer');
+    return $this->GetParentContainer();
   }
 
 
@@ -992,6 +1074,14 @@ EOQRY;
   }
 
 
+  /**
+  * Checks whether this resource is a bind to an external resource
+  */
+  function IsExternal() {
+    return $this->_is_external;
+  }
+
+
   /**
   * Checks whether this resource actually exists, in the virtual sense, within the hierarchy
   */
@@ -1017,7 +1107,7 @@ EOQRY;
     if ( $this->collection->dav_name != $this->dav_name ) {
       return $this->collection->exists;
     }
-    $parent = $this->FetchParentContainer();
+    $parent = $this->GetParentContainer();
     return $parent->Exists();
   }
 
@@ -1088,6 +1178,24 @@ EOQRY;
   }
 
 
+  /**
+  * Returns the internal user_no for the principal for this resource
+  */
+  function user_no() {
+    if ( !isset($this->principal) ) $this->FetchPrincipal();
+    return $this->principal->user_no();
+  }
+
+
+  /**
+  * Returns the internal collection_id for this collection, or the collection containing this resource
+  */
+  function collection_id() {
+    if ( !isset($this->collection) ) $this->FetchCollection();
+    return $this->collection->collection_id;
+  }
+
+
   /**
   * Returns the database row for this resource
   */
@@ -1102,7 +1210,10 @@ EOQRY;
   */
   function unique_tag() {
     if ( isset($this->unique_tag) ) return $this->unique_tag;
-    if ( $this->IsPrincipal() && !isset($this->principal) ) $this->FetchPrincipal();
+    if ( $this->IsPrincipal() && !isset($this->principal) ) {
+      $this->FetchPrincipal();
+      $this->unique_tag = $this->principal->unique_tag();
+    }
     else if ( !$this->_is_collection && !isset($this->resource) ) $this->FetchResource();
 
     if ( $this->exists !== true || !isset($this->unique_tag) ) $this->unique_tag = '';
@@ -1202,7 +1313,8 @@ EOQRY;
     $acl[] = $this->BuildACE($xmldoc, pow(2,25) - 1, new XMLElement('property', new XMLElement('owner')) );
 
     $qry = new AwlQuery('SELECT dav_principal.dav_name, grants.* FROM grants JOIN dav_principal ON (to_principal=principal_id) WHERE by_collection = :collection_id OR by_principal = :principal_id ORDER BY by_collection',
-                                array( ':collection_id' => $this->collection->collection_id, ':principal_id' => $this->principal->principal_id ) );
+                                array( ':collection_id' => $this->collection->collection_id,
+                                       ':principal_id' => $this->principal->principal_id() ) );
     if ( $qry->Exec('DAVResource') && $qry->rows() > 0 ) {
       $by_collection = null;
       while( $grant = $qry->Fetch() ) {
@@ -1233,6 +1345,11 @@ EOQRY;
         return $this->collection->collection_id;
         break;
 
+      case 'principal_id':
+        if ( !isset($this->principal) ) $this->FetchPrincipal();
+        return $this->principal->principal_id();
+        break;
+
       case 'resourcetype':
         if ( isset($this->resourcetypes) ) {
           $this->resourcetypes = preg_replace('{^\s*<(.*)/>\s*$}', '$1', $this->resourcetypes);
@@ -1378,26 +1495,28 @@ EOQRY;
 
       case 'DAV::resourcetype':
         $resourcetypes = $prop->NewElement('resourcetype' );
-        $type_list = $this->GetProperty('resourcetype');
-        if ( !is_array($type_list) ) return true;
-//        dbg_error_log( 'DAVResource', ':ResourceProperty: "%s" are "%s".', $tag, implode(', ',$type_list) );
-        foreach( $type_list AS $k => $v ) {
-          if ( $v == '' ) continue;
-          $reply->NSElement( $resourcetypes, $v );
-        }
-        if ( $this->_is_binding ) {
-          $reply->NSElement( $resourcetypes, 'http://xmlns.davical.org/davical:webdav-binding' );
+        if ( $this->_is_collection ) {
+          $type_list = $this->GetProperty('resourcetype');
+          if ( !is_array($type_list) ) return true;
+  //        dbg_error_log( 'DAVResource', ':ResourceProperty: "%s" are "%s".', $tag, implode(', ',$type_list) );
+          foreach( $type_list AS $k => $v ) {
+            if ( $v == '' ) continue;
+            $reply->NSElement( $resourcetypes, $v );
+          }
+          if ( $this->_is_binding ) {
+            $reply->NSElement( $resourcetypes, 'http://xmlns.davical.org/davical:webdav-binding' );
+          }
         }
         break;
 
       case 'DAV::getlastmodified':
-        /** peculiarly, it seems that getlastmodified is HTTP Date format! */
+        /** getlastmodified is HTTP Date format: i.e. the Last-Modified header in response to a GET */
         $reply->NSElement($prop, $tag, ISODateToHTTPDate($this->GetProperty('modified')) );
         break;
 
       case 'DAV::creationdate':
-        /** bizarrely, it seems that creationdate is ISO8601 format */
-        $reply->NSElement($prop, $tag, DateToISODate($this->GetProperty('created')) );
+        /** creationdate is ISO8601 format */
+        $reply->NSElement($prop, $tag, DateToISODate($this->GetProperty('created'), true) );
         break;
 
       case 'DAV::getcontentlength':
@@ -1427,9 +1546,9 @@ EOQRY;
         break;
 
       case 'DAV::owner':
-        // After a careful reading of RFC3744 we see that this must be the principal-URL of the owner
-        $owner_url = ( isset($this->_is_binding) && $this->_is_binding ? $this->collection->bind_owner_url : $this->principal_url() );
-        $reply->DAVElement( $prop, 'owner', $reply->href( $owner_url ) );
+        // The principal-URL of the owner
+        if ( !isset($this->principal) ) $this->FetchPrincipal();
+        $reply->DAVElement( $prop, 'owner', $reply->href( $this->principal->url() ) );
         break;
 
       // Empty tag responses.
@@ -1511,10 +1630,10 @@ EOQRY;
         break;
 
       case 'DAV::current-user-principal':
-        $prop->NewElement('current-user-principal', $reply->href( $request->principal->principal_url ) );
+        $prop->NewElement('current-user-principal', $reply->href( $request->principal->url() ) );
         break;
 
-      case 'SOME-DENIED-PROPERTY':  /** @todo indicating the style for future expansion */
+      case 'SOME-DENIED-PROPERTY':  /** indicating the style for future expansion */
         $denied[] = $reply->Tag($tag);
         break;
 
@@ -1531,6 +1650,7 @@ EOQRY;
       case 'urn:ietf:params:xml:ns:carddav:address-data':
       case 'urn:ietf:params:xml:ns:caldav:calendar-data':
         if ( $this->_is_collection ) return false;
+        if ( !isset($c->sync_resource_data_ok) || $c->sync_resource_data_ok == false ) return false;
         if ( !isset($this->resource) ) $this->FetchResource();
         $reply->NSElement($prop, $tag, $this->resource->caldav_data );
         break;
