@@ -45,12 +45,18 @@ class HTTPAuthSession {
   /**#@-*/
 
   /**
-  * The constructor, which just calls the actual type configured
+  * The constructor, which just calls the type supplied or configured
   */
   function HTTPAuthSession() {
     global $c;
 
-    if ( isset($c->http_auth_mode) && $c->http_auth_mode == "Digest" ) {
+    if ( ! empty($_SERVER['PHP_AUTH_DIGEST'])) {
+      $this->DigestAuthSession();
+    }
+    else if ( isset($_SERVER["AUTHORIZATION"]) || isset($_SERVER['PHP_AUTH_USER']) ) {
+      $this->BasicAuthSession();
+    }
+    else if ( isset($c->http_auth_mode) && $c->http_auth_mode == "Digest" ) {
       $this->DigestAuthSession();
     }
     else {
@@ -66,14 +72,22 @@ class HTTPAuthSession {
   function AuthFailedResponse( $auth_header = "" ) {
     global $c;
     if ( $auth_header == "" ) {
-      $auth_header = sprintf( 'WWW-Authenticate: Basic realm="%s"', $c->system_name);
+      $auth_realm = $c->system_name;
+      if ( isset($c->per_principal_realm) && $c->per_principal_realm ) {
+        $principal_name = preg_replace( '{^/(.*?)/.*$}', '$1', $_SERVER["PATH_INFO"]);
+        if ( $principal_name != $_SERVER["PATH_INFO"] ) {
+          $auth_realm .= ' - ' . $principal_name;
+        }
+      }
+      dbg_error_log( "HTTPAuth", ":AuthFailedResponse Requesting authentication in the '%s' realm", $auth_realm );
+      $auth_header = sprintf( 'WWW-Authenticate: Basic realm="%s"', $auth_realm );
     }
 
     header('HTTP/1.1 401 Unauthorized', true, 401 );
     header('Content-type: text/plain; ; charset="utf-8"' );
     header( $auth_header );
     echo 'Please log in for access to this system.';
-    dbg_error_log( "HTTPAuth", ":Session: User is not authorised" );
+    dbg_error_log( "HTTPAuth", ":Session: User is not authorised: %s ", $_SERVER['REMOTE_ADDR'] );
     exit;
   }
 
@@ -120,23 +134,21 @@ class HTTPAuthSession {
     * Fall through to the normal PHP authentication variables.
     */
     if ( isset($_SERVER['PHP_AUTH_USER']) ) {
-      if ( $u = $this->CheckPassword( $_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'] ) ) {
+      if ( $p = $this->CheckPassword( $_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'] ) ) {
         /**
          * Maybe some external authentication didn't return false for an inactive
          * user, so we'll be pedantic here. 
          */
-        if ( $u->active ) {
-          $this->AssignSessionDetails($u);
+        if ( $p->user_active ) {
+          $this->AssignSessionDetails($p);
           return;
         }
       }
     }
 
     if ( isset($c->allow_unauthenticated) && $c->allow_unauthenticated ) {
-      $this->user_no = -1;
-      $this->username = 'guest';
-      $this->fullname = 'Unauthenticated User';
-      $this->email = 'invalid';
+      $this->AssignSessionDetails('unauthenticated');
+      $this->logged_in = false;
       return;
     }
 
@@ -162,30 +174,58 @@ class HTTPAuthSession {
   function DigestAuthSession() {
     global $c;
 
+    $realm = $c->system_name;
+    $opaque = $realm;
+    if ( isset($_SERVER['HTTP_USER_AGENT']) ) $opaque .= $_SERVER['HTTP_USER_AGENT'];
+    if ( isset($_SERVER['REMOTE_ADDR']) )     $opaque .= $_SERVER['REMOTE_ADDR'];
+    $opaque = sha1($opaque);
+    
     if ( ! empty($_SERVER['PHP_AUTH_DIGEST'])) {
       // analyze the PHP_AUTH_DIGEST variable
       if ( $data = $this->ParseDigestHeader($_SERVER['PHP_AUTH_DIGEST']) ) {
-        // generate the valid response
-        $user_password = "Don't be silly! Why would a user have a password like this!!?";
-        /**
-        * @todo At this point we need to query the database for something fitting
-        * either strategy (A) or (B) above, in order to set $user_password to
-        * something useful!
-        */
-        $A1 = md5($data['username'] . ':' . $c->system_name . ':' . $user_password);
-        $A2 = md5($_SERVER['REQUEST_METHOD'].':'.$data['uri']);
-        $valid_response = md5($A1.':'.$data['nonce'].':'.$data['nc'].':'.$data['cnonce'].':'.$data['qop'].':'.$A2);
 
-        if ( $data['response'] == $valid_response ) {
-          $this->AssignSessionDetails($u);
-          return;
+        if ( $data['uri'] != $_SERVER['REQUEST_URI'] ) {
+          dbg_error_log( "ERROR", " DigestAuth: WTF! URI is '%s' and request URI is '%s'!?!" );
+          $this->AuthFailedResponse();
+          // Does not return
+        }
+        
+        // generate the valid response
+        $test_user = new Principal('username', $data['username']);
+
+        if ( preg_match( '{\*(Digest)?\*(.*)}', $test_user->password, $matches ) ) {
+          if ( $matches[1] == 'Digest' )
+            $A1 = $matches[2];
+          else {
+//            dbg_error_log( "HTTPAuth", "Constructing A1 from md5(%s:%s:%s)", $data['username'], $realm, $matches[2] );
+            $A1 = md5($data['username'] . ':' . $realm . ':' . $matches[2]);
+          }
+          $A2 = md5($_SERVER['REQUEST_METHOD'].':'.$data['uri']);
+          $auth_string = $A1.':'.$data['nonce'].':'.$data['nc'].':'.$data['cnonce'].':'.$data['qop'].':'.$A2;
+//          dbg_error_log( "HTTPAuth", "DigestAuthString: %s", $auth_string);
+          $valid_response = md5($auth_string);
+//          dbg_error_log( "HTTPAuth", "DigestResponse: %s", $valid_response);
+          
+          if ( $data['response'] == $valid_response ) {
+            $this->AssignSessionDetails($test_user);
+//            dbg_error_log( "HTTPAuth", "Success!!!" );
+            return;
+          }
+        }
+        else {
+          // Their account is not configured for Digest auth so we need to use Basic.
+          $this->AuthFailedResponse();
+          // Does not return
         }
       }
     }
 
-    $nonce = uniqid();
-    $opaque = md5($c->system_name);
-    $this->AuthFailedResponse( sprintf('WWW-Authenticate: Digest realm="%s", qop="auth,auth-int", nonce="%s", opaque="%s"', $c->system_name, $nonce, $opaque ) );
+    $nonce = sha1(uniqid('',true));
+    $authheader = sprintf('WWW-Authenticate: Digest realm="%s", qop="auth", nonce="%s", opaque="%s", algorithm="MD5"',
+                                     $realm, $nonce, $opaque );
+    dbg_error_log( "HTTPAuth", $authheader );
+    $this->AuthFailedResponse( $authheader );
+    // Does not return
   }
 
 
@@ -198,16 +238,30 @@ class HTTPAuthSession {
     $needed_parts = array('nonce'=>1, 'nc'=>1, 'cnonce'=>1, 'qop'=>1, 'username'=>1, 'uri'=>1, 'response'=>1);
     $data = array();
 
-    preg_match_all('@(\w+)=(?:([\'"])([^\2]+)\2|([^\s,]+))@', $auth_header, $matches, PREG_SET_ORDER);
-
+    preg_match_all('{(\w+)="([^"]+)"}', $auth_header, $matches, PREG_SET_ORDER);
     foreach ($matches as $m) {
-      $data[$m[1]] = $m[3] ? $m[3] : $m[4];
+//      dbg_error_log( "HTTPAuth", 'Match: "%s"', $m[0] );
+      $data[$m[1]] = $m[2];
       unset($needed_parts[$m[1]]);
+      dbg_error_log( "HTTPAuth", 'Received: %s: %s', $m[1], $m[2] );
     }
 
+    preg_match_all('{(\w+)=([^" ,]+)}', $auth_header, $matches, PREG_SET_ORDER);
+    foreach ($matches as $m) {
+//      dbg_error_log( "HTTPAuth", 'Match: "%s"', $m[0] );
+      $data[$m[1]] = $m[2];
+      unset($needed_parts[$m[1]]);
+      dbg_error_log( "HTTPAuth", 'Received: %s: %s', $m[1], $m[2] );
+    }
+
+    
+    @dbg_error_log( "HTTPAuth", 'Received: nonce: %s, nc: %s, cnonce: %s, qop: %s, username: %s, uri: %s, response: %s',
+        $data['nonce'], $data['nc'], $data['cnonce'], $data['qop'], $data['username'], $data['uri'], $data['response']
+      );
     return $needed_parts ? false : $data;
   }
 
+  
   /**
   * CheckPassword does all of the password checking and
   * returns a user record object, or false if it all ends in tears.
@@ -215,6 +269,9 @@ class HTTPAuthSession {
   function CheckPassword( $username, $password ) {
     global $c;
 
+    if(isset($c->login_append_domain_if_missing) && $c->login_append_domain_if_missing && !preg_match('/@/',$username))
+      $username.='@'.$c->domain_name;
+
     if ( isset($c->authenticate_hook) && isset($c->authenticate_hook['call']) && function_exists($c->authenticate_hook['call']) ) {
       /**
       * The authenticate hook needs to:
@@ -239,15 +296,16 @@ class HTTPAuthSession {
       }
     }
 
-    if ( $usr = getUserByName($username) ) {
-      dbg_error_log( "BasicAuth", ":CheckPassword: Name:%s, Pass:%s, File:%s, Active:%s", $username, $password, $usr->password, ($usr->active?'Yes':'No') );
-      if ( $usr->active && session_validate_password( $password, $usr->password ) ) {
-        return $usr;
+    if ( $principal = new Principal('username', $username) ) {
+      if ( isset($c->dbg['password']) ) dbg_error_log( "password", ":CheckPassword: Name:%s, Pass:%s, File:%s, Active:%s", $username, $password, $principal->password, ($principal->user_active?'Yes':'No') );
+      if ( $principal->user_active && session_validate_password( $password, $principal->password ) ) {
+        return $principal;
       }
     }
     return false;
   }
 
+  
   /**
   * Checks whether a user is allowed to do something.
   *
@@ -280,20 +338,26 @@ class HTTPAuthSession {
   * Internal function used to assign the session details to a user's new session.
   * @param object $u The user+session object we (probably) read from the database.
   */
-  function AssignSessionDetails( $u ) {
-    if ( !isset($u->principal_id) ) {
-      // If they don't have a principal_id set then we should re-read from our local database
-      $qry = new AwlQuery('SELECT * FROM dav_principal WHERE username = :username', array(':username' => $u->username) );
-      if ( $qry->Exec() && $qry->rows() == 1 ) {
-        $u = $qry->Fetch();
-      }
+  function AssignSessionDetails( $principal ) {
+    if ( is_string($principal) ) $principal = new Principal('username',$principal);
+    if ( get_class($principal) != 'Principal' ) {
+      $principal = new Principal('username',$principal->username);
     }
 
     // Assign each field in the selected record to the object
-    foreach( $u AS $k => $v ) {
+    foreach( $principal AS $k => $v ) {
       $this->{$k} = $v;
     }
-
+    if ( !get_class($principal) == 'Principal' ) {
+      throw new Exception('HTTPAuthSession::AssignSessionDetails could not find a Principal object');
+    }
+    $this->username = $principal->username();
+    $this->user_no  = $principal->user_no();
+    $this->principal_id = $principal->principal_id();
+    $this->email = $principal->email();
+    $this->dav_name = $principal->dav_name();
+    $this->principal = $principal;
+    
     $this->GetRoles();
     $this->logged_in = true;
     if ( function_exists("awl_set_locale") && isset($this->locale) && $this->locale != "" ) {
