@@ -11,27 +11,27 @@
 
 if ( !class_exists('DateTime') ) return;
 
-$rrule_expand_limit = array(
-  'YEARLY'  => array( 'bymonth' => 'expand', 'byweekno' => 'expand', 'byyearday' => 'expand', 'bymonthday' => 'expand',
-                      'byday' => 'expand', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' ),
-  'MONTHLY' => array( 'bymonth' => 'limit', 'bymonthday' => 'expand',
-                      'byday' => 'expand', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' ),
-  'WEEKLY'  => array( 'bymonth' => 'limit',
-                      'byday' => 'expand', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' ),
-  'DAILY'   => array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
-                      'byday' => 'limit', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' ),
-  'HOURLY'  => array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
-                      'byday' => 'limit', 'byhour' => 'limit', 'byminute' => 'expand', 'bysecond' => 'expand' ),
-  'MINUTELY'=> array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
-                      'byday' => 'limit', 'byhour' => 'limit', 'byminute' => 'limit', 'bysecond' => 'expand' ),
-  'SECONDLY'=> array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
-                      'byday' => 'limit', 'byhour' => 'limit', 'byminute' => 'limit', 'bysecond' => 'limit' ),
-);
+/**
+* Try and extract something like "Pacific/Auckland" or "America/Indiana/Indianapolis" if possible, given
+* the VTIMEZONE component that is passed in.  This is much more complex than olson_from_tzstring since
+* we start to examine the rules and work out what actual timezone this might be.
+*/
+function olson_from_vtimezone( vComponent $vtz ) {
+  $tzid = $vtz->GetProperty('TZID');
+  if ( empty($tzid) ) $tzid = $vtz->GetProperty('TZID'); 
+  if ( !empty($tzid) ) {
+    $result = olson_from_tzstring($tzid);
+    if ( !empty($result) ) return $result;
+  }
 
-$rrule_day_numbers = array( 'SU' => 0, 'MO' => 1, 'TU' => 2, 'WE' => 3, 'TH' => 4, 'FR' => 5, 'SA' => 6 );
+  /**
+   * @todo: We'll do other stuff here, in due course...
+   */
+  return null;
+}
 
-define( 'DEBUG_RRULE', false );
 // define( 'DEBUG_RRULE', true);
+define( 'DEBUG_RRULE', false );
 
 /**
 * Wrap the DateTimeZone class to allow parsing some iCalendar TZID strangenesses
@@ -70,6 +70,130 @@ class RepeatRuleTimeZone extends DateTimeZone {
   }
 }
 
+/**
+ * Provide a useful way of dealing with RFC5545 duration strings of the form
+ * ^-?P(\dW)|((\dD)?(T(\dH)?(\dM)?(\dS)?)?)$
+ */
+class Rfc5545Duration {
+  private $epoch_seconds = null;
+  private $days = 0;
+  private $secs = 0;
+  private $as_text = '';
+
+  /**
+   * Construct a new Rfc5545Duration either from incoming seconds or a text string.
+   * @param mixed $in_duration
+   */
+  function __construct( $in_duration ) {
+    if ( is_integer($in_duration) ) {
+      $this->epoch_seconds = $in_duration;
+      $this->as_text = '';
+    }
+    else if ( gettype($in_duration) == 'string' ) {
+//      preg_match('{^-?P(\dW)|((\dD)?(T(\dH)?(\dM)?(\dS)?)?)$}i', $in_duration, $matches) ) {
+      $this->as_text = $in_duration;
+      $this->epoch_seconds = null;
+    }
+    else {
+//      fatal('Passed duration is neither numeric nor string!');
+    }
+  }
+
+  /**
+   * Return true if $this and $other are equal, false otherwise.
+   * @param Rfc5545Duration $other
+   * @return boolean
+   */
+  function equals( $other ) {
+    if ( $this == $other ) return true;
+    if ( $this->asSeconds() == $other->asSeconds() ) return true; 
+    return false; 
+  }
+
+  /**
+   * Returns the duration as epoch seconds.
+   */
+  function asSeconds() {
+    if ( !isset($this->epoch_seconds) ) {
+      if ( preg_match('{^(-?)P(\d+W)|((\d+D)?(T(\d+H)?(\d+M)?(\d+S)?)?)$}i', $subject, $matches) ) {
+        if ( isset($matches[2]) ) $this->days = ($matches[2] * 7);
+        else {
+          if ( isset($matches[4]) ) $this->days = $matches[4];
+          $this->secs = 0;
+          if ( isset($matches[6]) ) $this->secs += $matches[6] * 3600;
+          if ( isset($matches[7]) ) $this->days += $matches[7] * 60;
+          if ( isset($matches[8]) ) $this->days += $matches[8];
+        }
+        if ( $matches[1] == '-' ) {
+          $this->days *= -1;
+          $this->secs *= -1;
+        }
+        $this->epoch_seconds = ($this->days * 86400) + $this->secs;
+      }
+      else {
+        throw new Exception('Invalid epoch: "'+$this->as_text+"'");
+      }
+    }
+    return $this->epoch_seconds;
+  }
+
+  
+  /**
+   * Returns the duration as a text string of the form ^(-?)P(\d+W)|((\d+D)?(T(\d+H)?(\d+M)?(\d+S)?)?)$
+   * @return string The stringified stuff.
+   */
+  function __toString() {
+    if ( empty($this->as_text) ) {
+      $this->as_text = ($this->epoch_seconds < 0 ? '-P' : 'P');
+      $in_duration = abs($this->epoch_seconds);
+      if ( $in_duration >= 86400 ) {
+        $this->days = floor($in_duration / 86400);
+        $in_duration -= $this->days * 86400;
+        if ( $in_duration == 0 && ($this->days / 7) == floor($this->days / 7) ) {
+          $this->as_text .= ($this->days/7).'W';
+          return $this->as_text;
+        }
+        $this->as_text .= $this->days.'D';
+      }
+      if ( $in_duration > 0 ) {
+        $secs = $in_duration;
+        $this->as_text .= 'T';
+        $hours = floor($in_duration / 3600);
+        if ( $hours > 0 ) $this->as_text .= $hours . 'H';
+        $minutes = floor(($in_duration % 3600) / 60);
+        if ( $minutes > 0 ) $this->as_text .= $minutes . 'M';
+        $seconds = $in_duration % 60;
+        if ( $seconds > 0 ) $this->as_text .= $seconds . 'S';
+      }
+    }
+    return $this->as_text;
+  }
+
+
+  /**
+   * Factory method to return an Rfc5545Duration object from the difference
+   * between two dates.
+   * 
+   * This is flawed, at present: we should really localise both dates and work 
+   * out the difference in days, then localise the times and work out the difference
+   * between the clock times.  On the other hand we're replacing a quick and dirty
+   * hack that did it exactly the same way in the past, so we're not making things
+   * any *worse* and at least we're making it clear that it could be improved...
+   * 
+   * The problem strikes (as they all do) across DST boundaries.
+   * 
+   * @todo Improve this to calculate the days difference and then the clock time diff
+   * and work from there.
+   * 
+   * @param RepeatRuleDateTime $d1
+   * @param RepeatRuleDateTime $d2
+   * @return Rfc5545Duration
+   */
+  static function fromTwoDates( $d1, $d2 ) {
+    $diff = $d2->epoch() - $d1->epoch();
+    return new Rfc5545Duration($diff);
+  } 
+}
 
 /**
 * Wrap the DateTime class to make it friendlier to passing in random strings from iCalendar
@@ -84,16 +208,17 @@ class RepeatRuleDateTime extends DateTime {
   private $tzid;
   private $is_date;
 
-  public function __construct($date = null, $dtz = null) {
+  public function __construct($date = null, $dtz = null, $is_date = null ) {
     if ( !isset(self::$UTCzone) ) self::$UTCzone = new RepeatRuleTimeZone('UTC');
     $this->is_date = false;
+    if ( isset($is_date) ) $this->is_date = $is_date;
     if ( !isset($date) ) {
       $date = date('Ymd\THis');            
       // Floating
       $dtz = self::$UTCzone;
-      $this->tzid = null;
     }
-
+    $this->tzid = null;
+    
     if ( is_object($date) && method_exists($date,'GetParameterValue') ) {
       $tzid = $date->GetParameterValue('TZID');
       $actual_date = $date->Value();
@@ -110,8 +235,10 @@ class RepeatRuleDateTime extends DateTime {
       }
       if ( strlen($actual_date) == 8 ) {
         // We allow dates without VALUE=DATE parameter, but we don't create them like that
-        $this->is_date;
+        $this->is_date = true;
       }
+//      $value_type = $date->GetParameterValue('VALUE');
+//      if ( isset($value_type) && $value_type == 'DATE' ) $this->is_date = true;
       $date = $actual_date;
       if ( DEBUG_RRULE ) printf( "Date%s property%s: %s%s\n", ($this->is_date ? "" : "Time"),
               (isset($this->tzid) ? ' with timezone' : ''), $date,
@@ -167,7 +294,8 @@ class RepeatRuleDateTime extends DateTime {
     }
 
     parent::__construct($date, $dtz);
-
+    if ( isset($is_date) ) $this->is_date = $is_date;
+    
     return $this;
   }
 
@@ -182,19 +310,29 @@ class RepeatRuleDateTime extends DateTime {
   }
 
 
+  public function setAsFloat() {
+    unset($this->tzid);
+  }
+
+
   public function isFloating() {
     return !isset($this->tzid);
   }
 
   public function isDate() {
-    return !isset($this->is_date);
+    return $this->is_date;
+  }
+
+  
+  public function setAsDate() {
+    $this->is_date = true;
   }
 
   
   public function modify( $interval ) {
 //    print ">>$interval<<\n";
     if ( preg_match('{^(-)?P(([0-9-]+)W)?(([0-9-]+)D)?T?(([0-9-]+)H)?(([0-9-]+)M)?(([0-9-]+)S)?$}', $interval, $matches) ) {
-      $minus = $matches[1];
+      $minus = (isset($matches[1])?$matches[1]:'');
       $interval = '';
       if ( isset($matches[2]) && $matches[2] != '' ) $interval .= $minus . $matches[3] . ' weeks ';
       if ( isset($matches[4]) && $matches[4] != '' ) $interval .= $minus . $matches[5] . ' days ';
@@ -210,33 +348,57 @@ class RepeatRuleDateTime extends DateTime {
   }
 
 
+  /**
+   * Always returns a time localised to UTC.  Even floating times are converted to UTC
+   * using the server's currently configured PHP timezone.  Even dates will include a
+   * time, which will be non-zero if they were localised dates.
+   * 
+   * @see RepeatRuleDateTime::FloatOrUTC()
+   */
   public function UTC() {
     $gmt = clone($this);
-    if ( isset($this->tzid) && $this->tzid != 'UTC' ) {
-      $dtz = parent::getTimezone();
+    if ( $this->tzid != 'UTC' ) {
+      if ( isset($this->tzid)) {
+        $dtz = parent::getTimezone();
+      }
+      else {
+        $dtz = new DateTimeZone(date_default_timezone_get());
+      }
       $offset = 0 - $dtz->getOffset($gmt);
       $gmt->modify( $offset . ' seconds' );
     }
-    if ( $this->is_date ) return $gmt->format('Ymd');
     return $gmt->format('Ymd\THis\Z');
   }
 
 
-  public function FloatOrUTC() {
+  /**
+   * If this is a localised time then this will return the UTC equivalent.  If it is a 
+   * floating time, then you will just get the floating time.  If it is a date then it
+   * will be returned as a date.  Note that if it is a *localised* date then the answer
+   * will still be the UTC equivalent but only the date itself will be returned.
+   * 
+   * If return_floating_times is true then all dates will be returned as floating times
+   * and UTC will not be returned. 
+   * 
+   * @see RepeatRuleDateTime::UTC()
+   */
+  public function FloatOrUTC($return_floating_times = false) {
     $gmt = clone($this);
-    if ( isset($this->tzid) && $this->tzid != 'UTC' ) {
+    if ( !$return_floating_times && isset($this->tzid) && $this->tzid != 'UTC' ) {
       $dtz = parent::getTimezone();
       $offset = 0 - $dtz->getOffset($gmt);
       $gmt->modify( $offset . ' seconds' );
     }
     if ( $this->is_date ) return $gmt->format('Ymd');
-    $result = $gmt->format('Ymd\THis');
-    if ( isset($this->tzid) ) $result .= 'Z';
-    return $result;
+    if ( $return_floating_times ) return $gmt->format('Ymd\THis');
+    return $gmt->format('Ymd\THis') . (!$return_floating_times && isset($this->tzid) ? 'Z' : '');
   }
 
 
-  public function RFC5545() {
+  /**
+   * Returns the string following a property name for an RFC5545 DATE-TIME value.
+   */
+  public function RFC5545($return_floating_times = false) {
     $result = '';
     if ( isset($this->tzid) && $this->tzid != 'UTC' ) {
       $result = ';TZID='.$this->tzid;
@@ -246,7 +408,7 @@ class RepeatRuleDateTime extends DateTime {
     }
     else {
       $result .= ':' . $this->format('Ymd\THis');
-      if ( isset($this->tzid) && $this->tzid == 'UTC' ) {
+      if ( !$return_floating_times && isset($this->tzid) && $this->tzid == 'UTC' ) {
         $result .= 'Z';
       }
     }
@@ -254,11 +416,6 @@ class RepeatRuleDateTime extends DateTime {
   }
 
 
-  public function RFC5545Duration( $end_stamp ) {
-    return sprintf( 'PT%dM', intval(($end_stamp->epoch() - $this->epoch()) / 60) );
-  }
-
-
   public function setTimeZone( $tz ) {
     if ( is_string($tz) ) {
       $tz = new RepeatRuleTimeZone($tz);
@@ -269,6 +426,11 @@ class RepeatRuleDateTime extends DateTime {
   }
 
 
+  public function getTimeZone() {
+    return $this->tzid;
+  }
+
+
   function setDate( $year=null, $month=null, $day=null ) {
     if ( !isset($year) )  $year  = parent::format('Y');
     if ( !isset($month) ) $month = parent::format('m');
@@ -277,6 +439,21 @@ class RepeatRuleDateTime extends DateTime {
     return $this;
   }
 
+  function setYearDay( $yearday ) {
+    if ( $yearday > 0 ) {
+      $current_yearday = parent::format('z') + 1;
+    }
+    else {
+      $current_yearday = (parent::format('z') - (365 + parent::format('L')));
+    }
+    $diff = $yearday - $current_yearday;
+    if ( $diff < 0 ) $this->modify('-P'.-$diff.'D');
+    else if ( $diff > 0 ) $this->modify('P'.$diff.'D');
+//    printf( "Current: %d, Looking for: %d, Diff: %d, What we got: %s (%d,%d)\n", $current_yearday, $yearday, $diff,
+//                 parent::format('Y-m-d'), (parent::format('z')+1), ((parent::format('z') - (365 + parent::format('L')))) );
+    return $this;
+  }
+
   function year() {
     return parent::format('Y');
   }
@@ -307,6 +484,13 @@ class RepeatRuleDateTime extends DateTime {
 }
 
 
+/**
+ * This class is an implementation of RRULE parsing and expansion, as per RFC5545.  It should be reasonably
+ * complete, except that it does not handle changing the WKST - there may be a few errors in unusual rules
+ * also, but all of the common cases should be handled correctly.
+ *  
+ * @author Andrew McMillan <andrew@mcmillan.net.nz>
+ */
 class RepeatRule {
 
   private $base;
@@ -329,22 +513,29 @@ class RepeatRule {
   private $position;
   private $finished;
   private $current_base;
+  private $original_rule;
 
 
-  public function __construct( $basedate, $rrule ) {
-    $this->base = ( is_object($basedate) ? $basedate : new RepeatRuleDateTime($basedate) );
+  public function __construct( $basedate, $rrule, $is_date=null, $return_floating_times=false ) {
+    if ( $return_floating_times ) $basedate->setAsFloat();
+    $this->base = (is_object($basedate) ? $basedate : new RepeatRuleDateTime($basedate) );
+    $this->original_rule = $rrule;
 
     if ( DEBUG_RRULE ) {
-      printf( "Constructing RRULE based on: '%s', rrule: '%s'\n", $basedate, $rrule );
+      printf( "Constructing RRULE based on: '%s', rrule: '%s' (we float: %s)\n", $basedate, $rrule, ($return_floating_times?"yes":"no") );
     }
 
     if ( preg_match('{FREQ=([A-Z]+)(;|$)}', $rrule, $m) ) $this->freq = $m[1];
-    if ( preg_match('{UNTIL=([0-9TZ]+)(;|$)}', $rrule, $m) ) $this->until = new RepeatRuleDateTime($m[1]);
+
+    if ( preg_match('{UNTIL=([0-9TZ]+)(;|$)}', $rrule, $m) )
+      $this->until = new RepeatRuleDateTime($m[1],$this->base->getTimeZone(),$is_date);
     if ( preg_match('{COUNT=([0-9]+)(;|$)}', $rrule, $m) ) $this->count = $m[1];
     if ( preg_match('{INTERVAL=([0-9]+)(;|$)}', $rrule, $m) ) $this->interval = $m[1];
+
     if ( preg_match('{WKST=(MO|TU|WE|TH|FR|SA|SU)(;|$)}', $rrule, $m) ) $this->wkst = $m[1];
 
-    if ( preg_match('{BYDAY=(([+-]?[0-9]{0,2}(MO|TU|WE|TH|FR|SA|SU),?)+)(;|$)}', $rrule, $m) )  $this->byday = explode(',',$m[1]);
+    if ( preg_match('{BYDAY=(([+-]?[0-9]{0,2}(MO|TU|WE|TH|FR|SA|SU),?)+)(;|$)}', $rrule, $m) )
+      $this->byday = explode(',',$m[1]);
 
     if ( preg_match('{BYYEARDAY=([0-9,+-]+)(;|$)}', $rrule, $m) ) $this->byyearday = explode(',',$m[1]);
     if ( preg_match('{BYWEEKNO=([0-9,+-]+)(;|$)}', $rrule, $m) ) $this->byweekno = explode(',',$m[1]);
@@ -370,7 +561,7 @@ class RepeatRule {
     }
     $this->frequency_string = sprintf('+%d %s', $this->interval, $this->freq_name );
     if ( DEBUG_RRULE ) printf( "Frequency modify string is: '%s', base is: '%s'\n", $this->frequency_string, $this->base->format('c') );
-    $this->Start();
+    $this->Start($return_floating_times);
   }
 
 
@@ -379,9 +570,9 @@ class RepeatRule {
   }
 
 
-  public function Start() {
+  public function Start($return_floating_times=false) {
     $this->instances = array();
-    $this->GetMoreInstances();
+    $this->GetMoreInstances($return_floating_times);
     $this->rewind();
     $this->finished = false;
   }
@@ -392,24 +583,25 @@ class RepeatRule {
   }
 
 
-  public function next() {
+  public function next($return_floating_times=false) {
     $this->position++;
-    return $this->current();
+    return $this->current($return_floating_times);
   }
 
 
-  public function current() {
+  public function current($return_floating_times=false) {
     if ( !$this->valid() ) return null;
-    if ( !isset($this->instances[$this->position]) ) $this->GetMoreInstances();
+    if ( !isset($this->instances[$this->position]) ) $this->GetMoreInstances($return_floating_times);
     if ( !$this->valid() ) return null;
-    if ( DEBUG_RRULE ) printf( "Returning date from position %d: %s (%s)\n", $this->position, $this->instances[$this->position]->format('c'), $this->instances[$this->position]->UTC() );
+    if ( DEBUG_RRULE ) printf( "Returning date from position %d: %s (%s)\n", $this->position,
+                          $this->instances[$this->position]->format('c'), $this->instances[$this->position]->FloatOrUTC($return_floating_times) );
     return $this->instances[$this->position];
   }
 
 
-  public function key() {
+  public function key($return_floating_times=false) {
     if ( !$this->valid() ) return null;
-    if ( !isset($this->instances[$this->position]) ) $this->GetMoreInstances();
+    if ( !isset($this->instances[$this->position]) ) $this->GetMoreInstances($return_floating_times);
     if ( !isset($this->keys[$this->position]) ) {
       $this->keys[$this->position] = $this->instances[$this->position];
     }
@@ -422,14 +614,46 @@ class RepeatRule {
     return false;
   }
 
-
-  private function GetMoreInstances() {
-    global $rrule_expand_limit;
-
+  /**
+   * This function returns an array which lists the order of processing, and whether the processing is
+   * to expand or limit based on this component.
+   * 
+   * Note that yearly-byday and monthly-byday have special handling which is coded within the
+   * expand_byday() method
+   * @param $freq a string indicating the frequency.
+   */
+  private static function rrule_expand_limit( $freq ) {
+    switch( $freq ) {
+      case 'YEARLY':
+        return array( 'bymonth' => 'expand', 'byweekno' => 'expand', 'byyearday' => 'expand', 'bymonthday' => 'expand',
+                        'byday' => 'expand', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' );
+      case 'MONTHLY':
+        return array( 'bymonth' => 'limit', 'bymonthday' => 'expand',
+                        'byday' => 'expand', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' );
+      case 'WEEKLY':
+        return array( 'bymonth' => 'limit',
+                        'byday' => 'expand', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' );
+      case 'DAILY':
+        return array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
+                        'byday' => 'limit', 'byhour' => 'expand', 'byminute' => 'expand', 'bysecond' => 'expand' );
+      case 'HOURLY':
+        return array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
+                        'byday' => 'limit', 'byhour' => 'limit', 'byminute' => 'expand', 'bysecond' => 'expand' );
+      case 'MINUTELY':
+        return array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
+                        'byday' => 'limit', 'byhour' => 'limit', 'byminute' => 'limit', 'bysecond' => 'expand' );
+      case 'SECONDLY':
+        return array( 'bymonth' => 'limit', 'bymonthday' => 'limit',
+                        'byday' => 'limit', 'byhour' => 'limit', 'byminute' => 'limit', 'bysecond' => 'limit' );
+    }
+  }
+  
+  private function GetMoreInstances($return_floating_times=false) {
     if ( $this->finished ) return;
     $got_more = false;
     $loop_limit = 10;
     $loops = 0;
+    if ( $return_floating_times ) $this->base->setAsFloat();
     while( !$this->finished && !$got_more && $loops++ < $loop_limit ) {
       if ( !isset($this->current_base) ) {
         $this->current_base = clone($this->base);
@@ -437,12 +661,16 @@ class RepeatRule {
       else {
         $this->current_base->modify( $this->frequency_string );
       }
+      if ( $return_floating_times ) $this->current_base->setAsFloat();
       if ( DEBUG_RRULE ) printf( "Getting more instances from: '%s' - %d\n", $this->current_base->format('c'), count($this->instances) );
       $this->current_set = array( clone($this->current_base) );
-      foreach( $rrule_expand_limit[$this->freq] AS $bytype => $action ) {
-        if ( isset($this->{$bytype}) ) $this->{$action.'_'.$bytype}();
-        if ( !isset($this->current_set[0]) ) break;
+      foreach( self::rrule_expand_limit($this->freq) AS $bytype => $action ) {
+        if ( isset($this->{$bytype}) ) {
+          $this->{$action.'_'.$bytype}();          
+          if ( !isset($this->current_set[0]) ) break;
+        } 
       }
+  
       sort($this->current_set);
       if ( isset($this->bysetpos) ) $this->limit_bysetpos();
 
@@ -465,11 +693,24 @@ class RepeatRule {
     }
   }
 
-
+  
+  public static function rrule_day_number( $day ) {
+    switch( $day ) {
+      case 'SU': return 0;
+      case 'MO': return 1;
+      case 'TU': return 2;
+      case 'WE': return 3;
+      case 'TH': return 4;
+      case 'FR': return 5;
+      case 'SA': return 6;
+    }
+    return false;
+  }
+  
+  
   static public function date_mask( $date, $y, $mo, $d, $h, $mi, $s ) {
     $date_parts = explode(',',$date->format('Y,m,d,H,i,s'));
 
-    $tz = $date->getTimezone();
     if ( isset($y) || isset($mo) || isset($d) ) {
       if ( isset($y) ) $date_parts[0] = $y;
       if ( isset($mo) ) $date_parts[1] = $mo;
@@ -508,18 +749,31 @@ class RepeatRule {
     }
   }
 
-
+  private function expand_byyearday() {
+    $instances = $this->current_set;
+    $this->current_set = array();
+    $days_set = array();
+    foreach( $instances AS $k => $instance ) {
+      foreach( $this->byyearday AS $k => $yearday ) {
+        $on_yearday = clone($instance);
+        $on_yearday->setYearDay($yearday);
+        if ( isset($days_set[$on_yearday->UTC()]) ) continue;
+        $this->current_set[] = $on_yearday;
+        $days_set[$on_yearday->UTC()] = true;
+      }
+    }
+  }
+  
   private function expand_byday_in_week( $day_in_week ) {
-    global $rrule_day_numbers;
 
     /**
-    * @TODO: This should really allow for WKST, since if we start a series
+    * @todo This should really allow for WKST, since if we start a series
     * on (eg.) TH and interval > 1, a MO, TU, FR repeat will not be in the
     * same week with this code.
     */
     $dow_of_instance = $day_in_week->format('w'); // 0 == Sunday
     foreach( $this->byday AS $k => $weekday ) {
-      $dow = $rrule_day_numbers[$weekday];
+      $dow = self::rrule_day_number($weekday);
       $offset = $dow - $dow_of_instance;
       if ( $offset < 0 ) $offset += 7;
       $expanded = clone($day_in_week);
@@ -531,14 +785,13 @@ class RepeatRule {
 
 
   private function expand_byday_in_month( $day_in_month ) {
-    global $rrule_day_numbers;
 
     $first_of_month = $this->date_mask( clone($day_in_month), null, null, 1, null, null, null);
     $dow_of_first = $first_of_month->format('w'); // 0 == Sunday
     $days_in_month = cal_days_in_month(CAL_GREGORIAN, $first_of_month->format('m'), $first_of_month->format('Y'));
     foreach( $this->byday AS $k => $weekday ) {
       if ( preg_match('{([+-])?(\d)?(MO|TU|WE|TH|FR|SA|SU)}', $weekday, $matches ) ) {
-        $dow = $rrule_day_numbers[$matches[3]];
+        $dow = self::rrule_day_number($matches[3]);
         $first_dom = 1 + $dow - $dow_of_first;  if ( $first_dom < 1 ) $first_dom +=7;  // e.g. 1st=WE, dow=MO => 1+1-3=-1 => MO is 6th, etc.
         $whichweek = intval($matches[2]);
         if ( DEBUG_RRULE ) printf( "Expanding BYDAY(M) $weekday in month of %s\n", $first_of_month->format('c') );
@@ -572,7 +825,6 @@ class RepeatRule {
 
 
   private function expand_byday_in_year( $day_in_year ) {
-    global $rrule_day_numbers;
 
     $first_of_year = $this->date_mask( clone($day_in_year), null, 1, 1, null, null, null);
     $dow_of_first = $first_of_year->format('w'); // 0 == Sunday
@@ -580,7 +832,7 @@ class RepeatRule {
     foreach( $this->byday AS $k => $weekday ) {
       if ( preg_match('{([+-])?(\d)?(MO|TU|WE|TH|FR|SA|SU)}', $weekday, $matches ) ) {
         $expanded = clone($first_of_year);
-        $dow = $rrule_day_numbers[$matches[3]];
+        $dow = self::rrule_day_number($matches[3]);
         $first_doy = 1 + $dow - $dow_of_first;  if ( $first_doy < 1 ) $first_doy +=7;  // e.g. 1st=WE, dow=MO => 1+1-3=-1 => MO is 6th, etc.
         $whichweek = intval($matches[2]);
         if ( DEBUG_RRULE ) printf( "Expanding BYDAY(Y) $weekday from date %s\n", $instance->format('c') );
@@ -689,13 +941,11 @@ class RepeatRule {
   }
 
   private function limit_byday() {
-    global $rrule_day_numbers;
-
     $fmt_char = 'w';
     $instances = $this->current_set;
     $this->current_set = array();
     foreach( $this->byday AS $k => $weekday ) {
-      $dow = $rrule_day_numbers[$weekday];
+      $dow = self::rrule_day_number($weekday);
       foreach( $instances AS $k => $instance ) {
         if ( DEBUG_RRULE ) printf( "Limiting '$fmt_char' on '%s' => '%s' ?=? '%s' (%d) ? %s\n", $instance->format('c'), $instance->format($fmt_char), $weekday, $dow, ($instance->format($fmt_char) == $dow ? 'Yes' : 'No') );
         if ( $instance->format($fmt_char) == $dow ) $this->current_set[] = $instance;
@@ -741,7 +991,7 @@ require_once("vComponent.php");
 *
 * @return array An array keyed on the UTC dates, referring to the component
 */
-function rdate_expand( $dtstart, $property, $component, $range_end = null ) {
+function rdate_expand( $dtstart, $property, $component, $range_end = null, $is_date=null, $return_floating_times=false ) {
   $properties = $component->GetProperties($property);
   $expansion = array();
   foreach( $properties AS $p ) {
@@ -749,8 +999,9 @@ function rdate_expand( $dtstart, $property, $component, $range_end = null ) {
     $rdate = $p->Value();
     $rdates = explode( ',', $rdate );
     foreach( $rdates AS $k => $v ) {
-      $rdate = new RepeatRuleDateTime( $v, $timezone);
-      $expansion[$rdate->UTC()] = $component;
+      $rdate = new RepeatRuleDateTime( $v, $timezone, $is_date);
+      if ( $return_floating_times ) $rdate->setAsFloat();
+      $expansion[$rdate->FloatOrUTC($return_floating_times)] = $component;
       if ( $rdate > $range_end ) break;
     }
   }
@@ -768,7 +1019,7 @@ function rdate_expand( $dtstart, $property, $component, $range_end = null ) {
 *
 * @return array An array keyed on the UTC dates, referring to the component
 */
-function rrule_expand( $dtstart, $property, $component, $range_end ) {
+function rrule_expand( $dtstart, $property, $component, $range_end, $is_date=null, $return_floating_times=false ) {
   $expansion = array();
 
   $recur = $component->GetProperty($property);
@@ -777,24 +1028,24 @@ function rrule_expand( $dtstart, $property, $component, $range_end ) {
 
   $this_start = $component->GetProperty('DTSTART');
   if ( isset($this_start) ) {
-    $timezone = $this_start->GetParameterValue('TZID');
-    $this_start = new RepeatRuleDateTime($this_start->Value(),$timezone);
+    $this_start = new RepeatRuleDateTime($this_start);
   }
   else {
     $this_start = clone($dtstart);
   }
-
-//  print_r( $this_start );
-//  printf( "RRULE: %s\n", $recur );
-  $rule = new RepeatRule( $this_start, $recur );
+  if ( $return_floating_times ) $this_start->setAsFloat();
+  
+//  if ( DEBUG_RRULE ) print_r( $this_start );
+  if ( DEBUG_RRULE ) printf( "RRULE: %s (floating: %s)\n", $recur, ($return_floating_times?"yes":"no") );
+  $rule = new RepeatRule( $this_start, $recur, $is_date, $return_floating_times );
   $i = 0;
   $result_limit = 1000;
-  while( $date = $rule->next() ) {
-//    printf( "[%3d] %s\n", $i, $date->UTC() );
-    $expansion[$date->UTC()] = $component;
+  while( $date = $rule->next($return_floating_times) ) {
+//    if ( DEBUG_RRULE ) printf( "[%3d] %s\n", $i, $date->UTC() );
+    $expansion[$date->FloatOrUTC($return_floating_times)] = $component;
     if ( $i++ >= $result_limit || $date > $range_end ) break;
   }
-//  print_r( $expansion );
+//  if ( DEBUG_RRULE ) print_r( $expansion );
   return $expansion;
 }
 
@@ -810,7 +1061,8 @@ function rrule_expand( $dtstart, $property, $component, $range_end ) {
 *
 * @return vComponent The original vComponent, with the instances of the internal components expanded.
 */
-function expand_event_instances( $vResource, $range_start = null, $range_end = null ) {
+function expand_event_instances( $vResource, $range_start = null, $range_end = null, $return_floating_times=false ) {
+  global $c;
   $components = $vResource->GetComponents();
 
   if ( !isset($range_start) ) { $range_start = new RepeatRuleDateTime(); $range_start->modify('-6 weeks'); }
@@ -821,21 +1073,35 @@ function expand_event_instances( $vResource, $range_start = null, $range_end = n
   $instances = array();
   $expand = false;
   $dtstart = null;
+  $is_date = false;
+  $has_repeats = false;
+  $dtstart_type = 'DTSTART';
   foreach( $components AS $k => $comp ) {
     if ( $comp->GetType() != 'VEVENT' && $comp->GetType() != 'VTODO' && $comp->GetType() != 'VJOURNAL' ) {
       if ( $comp->GetType() != 'VTIMEZONE' ) $new_components[] = $comp;
       continue;
     }
     if ( !isset($dtstart) ) {
-      $dtstart = $comp->GetProperty('DTSTART');
-      $dtstart = new RepeatRuleDateTime( $dtstart );
-      $instances[$dtstart->UTC()] = $comp;
+      $dtstart_prop = $comp->GetProperty($dtstart_type);
+      if ( !isset($dtstart_prop) && $comp->GetType() != 'VTODO' ) {
+        $dtstart_type = 'DUE';
+        $dtstart_prop = $comp->GetProperty($dtstart_type);
+      }
+      if ( !isset($dtstart_prop) ) continue;
+      $dtstart = new RepeatRuleDateTime( $dtstart_prop );
+      if ( $return_floating_times ) $dtstart->setAsFloat();
+      if ( DEBUG_RRULE ) printf( "Component is: %s (floating: %s)\n", $comp->GetType(), ($return_floating_times?"yes":"no") );
+      $is_date = $dtstart->isDate();
+      $instances[$dtstart->FloatOrUTC($return_floating_times)] = $comp;
+      $rrule = $comp->GetProperty('RRULE');
+      $has_repeats = isset($rrule);
     }
     $p = $comp->GetProperty('RECURRENCE-ID');
     if ( isset($p) && $p->Value() != '' ) {
       $range = $p->GetParameterValue('RANGE');
       $recur_utc = new RepeatRuleDateTime($p);
-      $recur_utc = $recur_utc->UTC();
+      if ( $is_date ) $recur_utc->setAsDate();
+      $recur_utc = $recur_utc->FloatOrUTC($return_floating_times);
       if ( isset($range) && $range == 'THISANDFUTURE' ) {
         foreach( $instances AS $k => $v ) {
           if ( DEBUG_RRULE ) printf( "Removing overridden instance at: $k\n" );
@@ -852,14 +1118,14 @@ function expand_event_instances( $vResource, $range_start = null, $range_end = n
       $p =  $comp->GetProperty('UID');
       $uid = ( isset($p) ? $p->Value() : 'not set');
       printf( "Processing event '%s' with UID '%s' starting on %s\n",
-                 $summary, $uid, $dtstart->UTC() );      
+                 $summary, $uid, $dtstart->FloatOrUTC($return_floating_times) );      
       print( "Instances at start");
       foreach( $instances AS $k => $v ) {
         print ' : '.$k;        
       }
       print "\n";
     }
-    $instances = array_merge( $instances, rrule_expand($dtstart, 'RRULE', $comp, $range_end) );
+    $instances += rrule_expand($dtstart, 'RRULE', $comp, $range_end, null, $return_floating_times);
     if ( DEBUG_RRULE ) {
       print( "After rrule_expand");
       foreach( $instances AS $k => $v ) {
@@ -867,7 +1133,7 @@ function expand_event_instances( $vResource, $range_start = null, $range_end = n
       }
       print "\n";
     }
-    $instances = array_merge( $instances, rdate_expand($dtstart, 'RDATE', $comp, $range_end) );
+    $instances += rdate_expand($dtstart, 'RDATE', $comp, $range_end, null, $return_floating_times);
     if ( DEBUG_RRULE ) {
       print( "After rdate_expand");
       foreach( $instances AS $k => $v ) {
@@ -875,7 +1141,7 @@ function expand_event_instances( $vResource, $range_start = null, $range_end = n
       }
       print "\n";
     }
-    foreach ( rdate_expand($dtstart, 'EXDATE', $comp, $range_end) AS $k => $v ) {
+    foreach ( rdate_expand($dtstart, 'EXDATE', $comp, $range_end, null, $return_floating_times) AS $k => $v ) {
       unset($instances[$k]);
     }
     if ( DEBUG_RRULE ) {
@@ -890,54 +1156,111 @@ function expand_event_instances( $vResource, $range_start = null, $range_end = n
   $last_duration = null;
   $early_start = null;
   $new_components = array();
-  $start_utc = $range_start->UTC();
-  $end_utc = $range_end->UTC();
+  $start_utc = $range_start->FloatOrUTC($return_floating_times);
+  $end_utc = $range_end->FloatOrUTC($return_floating_times);
   foreach( $instances AS $utc => $comp ) {
-    if ( $utc > $end_utc ) break;
+    if ( $utc > $end_utc ) {
+      if ( DEBUG_RRULE ) printf( "We're done: $utc is out of the range.\n");
+      break;      
+    }
 
     $end_type = ($comp->GetType() == 'VTODO' ? 'DUE' : 'DTEND');
     $duration = $comp->GetProperty('DURATION');
     if ( !isset($duration) || $duration->Value() == '' ) {
-      $instance_start = $comp->GetProperty('DTSTART');
+      $instance_start = $comp->GetProperty($dtstart_type);
       $dtsrt = new RepeatRuleDateTime( $instance_start );
+      if ( $return_floating_times ) $dtsrt->setAsFloat();
       $instance_end = $comp->GetProperty($end_type);
       if ( isset($instance_end) ) {
         $dtend = new RepeatRuleDateTime( $instance_end );
-        $duration = $dtstart->RFC5545Duration( $dtend );
+        $duration = Rfc5545Duration::fromTwoDates($dtsrt, $dtend);
       }
       else {
         if ( $instance_start->GetParameterValue('VALUE') == 'DATE' ) {
-          $duration = 'P1D';
+          $duration = new Rfc5545Duration('P1D');
         }
         else {
-          $duration = 'P0D';  // For clarity
+          $duration = new Rfc5545Duration(0);
         }
       }
     }
     else {
-      $duration = $duration->Value();
+      $duration = new Rfc5545Duration($duration->Value());
     }
 
     if ( $utc < $start_utc ) {
-      if ( isset($early_start) && isset($last_duration) && $duration == $last_duration) {
-        if ( $utc < $early_start ) continue;
+      if ( isset($early_start) && isset($last_duration) && $duration->equals($last_duration) ) {
+        if ( $utc < $early_start ) {
+          if ( DEBUG_RRULE ) printf( "Next please: $utc is before $early_start and before $start_utc.\n");
+          continue;
+        }
       }
       else {
         /** Calculate the latest possible start date when this event would overlap our range start */
         $latest_start = clone($range_start);
         $latest_start->modify('-'.$duration);
-        $early_start = $latest_start->UTC();
+        $early_start = $latest_start->FloatOrUTC($return_floating_times);
         $last_duration = $duration;
-        if ( $utc < $early_start ) continue;
+        if ( $utc < $early_start ) {
+          if ( DEBUG_RRULE ) printf( "Another please: $utc is before $early_start and before $start_utc.\n");
+          continue;
+        }
       }
     }
     $component = clone($comp);
-    $component->ClearProperties( array('DTSTART'=> true, 'DUE' => true, 'DTEND' => true) );
-    $component->AddProperty('DTSTART', $utc );
+    if ( isset($c->expanded_instances_include_rrule) ) {
+      $component->ClearProperties( array('DTSTART'=> true, 'DUE' => true, 'DTEND' => true ) );
+    }
+    else {
+      $component->ClearProperties( array('DTSTART'=> true, 'DUE' => true, 'DTEND' => true,
+                                       'RRULE' => true, 'RDATE' => true, 'EXDATE' => true) );
+    }
+    $component->AddProperty('DTSTART', $utc, ($is_date ? array('VALUE' => 'DATE') : null) );
     $component->AddProperty('DURATION', $duration );
-    $new_components[] = $component;
+    if ( $has_repeats && $dtstart->FloatOrUTC($return_floating_times) != $utc )
+      $component->AddProperty('RECURRENCE-ID', $utc, ($is_date ? array('VALUE' => 'DATE') : null) );
+    $new_components[$utc] = $component;
   }
 
+  // Add overriden instances
+  foreach( $components AS $k => $comp ) {
+    $p = $comp->GetProperty('RECURRENCE-ID');
+    if ( isset($p) && $p->Value() != '') {
+      if ( !isset($new_components[$p->Value()]) ) {
+        // The component we're replacing is outside the range.  Unless the replacement
+        // is *in* the range we will move along to the next one.
+        $dtstart_prop = $comp->GetProperty($dtstart_type);
+        if ( !isset($dtstart_prop) ) continue;  // No start: no expansion.  Note that we consider 'DUE' to be a start if DTSTART is missing
+        $dtstart = new RepeatRuleDateTime( $dtstart_prop );
+        $is_date = $dtstart->isDate();  
+        if ( $return_floating_times ) $dtstart->setAsFloat();
+        $dtstart = $dtstart->FloatOrUTC($return_floating_times);
+        if ( $dtstart > $end_utc ) continue; // Start after end of range, skip it
+        
+        $end_type = ($comp->GetType() == 'VTODO' ? 'DUE' : 'DTEND');
+        $duration = $comp->GetProperty('DURATION');
+        if ( !isset($duration) || $duration->Value() == '' ) {
+          $instance_end = $comp->GetProperty($end_type);
+          if ( isset($instance_end) ) {
+            $dtend = new RepeatRuleDateTime( $instance_end );
+            if ( $return_floating_times ) $dtend->setAsFloat();
+            $dtend = $dtend->FloatOrUTC($return_floating_times); 
+          }
+          else {
+            $dtend = $dtstart  + ($is_date ? $dtstart + 86400 : 0 );
+          }
+        }
+        else {
+          $duration = new Rfc5545Duration($duration->Value());
+          $dtend = $dtstart + $duration->asSeconds();
+        }
+        if ( $dtend < $start_utc ) continue; // End before start of range: skip that too.
+      }
+      if ( DEBUG_RRULE ) printf( "Replacing overridden instance at %s\n", $p->Value());
+      $new_components[$p->Value()] = $comp;
+    }
+  }
+  
   $vResource->SetComponents($new_components);
 
   return $vResource;
