@@ -48,18 +48,23 @@ if ( ! ( $dav_resource->resource_id() > 0 ) ) {
 $qry = new AwlQuery();
 $qry->Begin();
 
-if ( $dav_resource->IsBinding() ) {
-  $params = array( ':dav_name' => $dav_resource->dav_name() );
-
-  if ( $qry->QDo("DELETE FROM dav_binding WHERE dav_name = :dav_name", $params )
-    && $qry->Commit() ) {
-    @dbg_error_log( "DELETE", "DELETE: Binding: %d, ETag: %s, Path: %s", $session->user_no, $request->etag_if_match, $request->path);
-    $request->DoResponse( 204 );
+if ( $dav_resource->IsCollection() ) {
+  if ( $dav_resource->IsBinding() ) {
+    $params = array( ':dav_name' => $dav_resource->dav_name() );
+  
+    if ( $qry->QDo("DELETE FROM dav_binding WHERE dav_name = :dav_name", $params )
+      && $qry->Commit() ) {
+      @dbg_error_log( "DELETE", "DELETE: Binding: %d, ETag: %s, Path: %s", $session->user_no, $request->etag_if_match, $request->path);
+      $request->DoResponse( 204 );
+    }
   }
-}
-else if ( $dav_resource->IsCollection() ) {
-  if ( delete_collection( $dav_resource->resource_id() ) && $qry->Commit() ) {
-    $request->DoResponse( 204 );
+  else {
+    if ( delete_collection( $dav_resource->resource_id() ) && $qry->Commit() ) {
+      // Uncache anything to do with the collection
+      $cache = getCacheInstance();
+      $cache->delete( 'collection-'.$dav_resource->dav_name(), null );
+      $request->DoResponse( 204 );
+    }
   }
 }
 else {
@@ -69,6 +74,10 @@ else {
 
   $params = array( ':dav_id' => $dav_resource->resource_id() );
 
+  // We need to serialise access to this process just for this collection
+  $cache = getCacheInstance();
+  $myLock = $cache->acquireLock('collection-'.$dav_resource->parent_path());
+
   if ( $qry->QDo("SELECT write_sync_change(collection_id, 404, caldav_data.dav_name) FROM caldav_data WHERE dav_id = :dav_id", $params )
     && $qry->QDo("DELETE FROM property WHERE dav_name = (SELECT dav_name FROM caldav_data WHERE dav_id = :dav_id)", $params )
     && $qry->QDo("DELETE FROM locks WHERE dav_name = (SELECT dav_name FROM caldav_data WHERE dav_id = :dav_id)", $params )
@@ -78,8 +87,12 @@ else {
     if ( function_exists('log_caldav_action') ) {
       log_caldav_action( 'DELETE', $dav_resource->GetProperty('uid'), $dav_resource->GetProperty('user_no'), $dav_resource->GetProperty('collection_id'), $request->path );
     }
+    $cache->delete( 'collection-'.$dav_resource->parent_path(), null );
+    $cache->releaseLock($myLock);
     $request->DoResponse( 204 );
   }
+  $cache->releaseLock($myLock);
+
 }
 
 $request->DoResponse( 500 );
