@@ -154,7 +154,7 @@ function handle_subaction( $subaction ) {
 }
 
 function principal_editor() {
-  global $id, $can_write_principal, $session;
+  global $c, $id, $can_write_principal, $session;
   $editor = new Editor(translate('Principal'), 'dav_principal');
   
   $editor->SetLookup( 'date_format_type', "SELECT 'E', 'European' UNION SELECT 'U', 'US Format' UNION SELECT 'I', 'ISO Format'" );
@@ -475,7 +475,11 @@ function group_memberships_browser() {
 function group_row_editor() {
   global $c, $id, $editor, $can_write_principal;
   $grouprow = new Editor("Group Members", "group_member");
-  $grouprow->SetLookup( 'member_id', 'SELECT principal_id, coalesce(displayname,fullname,username) FROM dav_principal WHERE principal_id NOT IN (SELECT member_id FROM group_member WHERE group_id = '.$id.') AND principal_id != '.$id);
+  $sql = 'SELECT principal_id, coalesce(displayname,fullname,username) FROM dav_principal ';
+  $sql .= 'WHERE principal_id NOT IN (SELECT member_id FROM group_member WHERE group_id = '.$id.') ';
+  $sql .= 'AND principal_id != '.$id;
+  $sql .= 'ORDER BY 2';
+  $grouprow->SetLookup( 'member_id', $sql);
   $grouprow->SetSubmitName( 'savegrouprow' );
 
   if ( $can_write_principal ) {
@@ -687,9 +691,8 @@ function ticket_row_editor() {
 
   $ticketrow = new Editor("Tickets", "access_ticket");
   $ticketrow->SetSubmitName( 'ticketrow' );
-  dbg_error_log( "ERROR", "Creating ticketrow editor: %s - %s", $can_write_principal, $ticketrow->IsSubmit());
   if ( $can_write_principal && $ticketrow->IsSubmit() ) {
-
+    
     $username = $editor->Value('username');
     $ugly_path = $_POST['target'];
     if ( $ugly_path == '/'.$username || $ugly_path == '/'.$username.'/' ) {
@@ -716,10 +719,10 @@ function ticket_row_editor() {
         return $ticketrow;
       }
     }
-      
+    
     $_POST['dav_owner_id'] = $id;
     $_POST['target_collection_id'] = $target_collection;
-    $ticket_id = clean_by_regex($_POST['ticket_id'], '/[A-Za-z0-9]+/');
+    $ticket_id = check_by_regex($_POST['ticket_id'], '/[A-Za-z0-9]+/');
     $ticketrow->SetWhere( 'dav_owner_id='.$id.' AND ticket_id='.AwlQuery::quote($ticket_id));
     if ( isset($_POST['ticket_privileges']) ) {
       $privilege_bitpos = array_flip($privilege_names);
@@ -966,7 +969,7 @@ if ( isset($id) && $id > 0 ) {
   
   $page_elements[] = bindings_to_other_browser();
   if ( isset($delete_bind_in_confirmation_required) ) $page_elements[] = confirm_delete_bind_in($delete_bind_in_confirmation_required);  
-
+  
   $page_elements[] = bindings_to_us_browser();
   if ( isset($delete_binding_confirmation_required) ) $page_elements[] = confirm_delete_binding($delete_binding_confirmation_required);
 }
